/**
 * This is an example plugin, it demonstrates how to add a custom event handler
 * in order to handle a custom inbound message.
 */
function ExamplePlugin() {
    this.version = "1.0.0";
    this.name = "Example Plugin";
    this.description = "This is an example plugin, demonstrating how to integrate with OmniScript to add custom events and handlers.";
    this.author = "Michael Powers <mpowers@scriptel.com>";
}
/**
 * This method handles inbound messages from the client.
 * @param message {object} The incoming message from the client.
 * @param connection {WebSocketConnection} The web socket connection to the client.
 * @param state {DeviceState} The state of the device the client is connected to, false if no device leased.
 * @param lease {DeviceLease} Information about the device the client has leased, false if no device leased.
 * @param simulateInput {function} Function that allows you to simulate incoming messages from the client.
 * @return {boolean} True if the inbound message has been handled and should be ignored, false otherwise.
 */
ExamplePlugin.prototype.handleInboundMessage = function(message, connection, state, lease, simulateInput) {
    if(message._class==="ExamplePluginRequest") {
        connection.sendUTF(JSON.stringify({"_class":"ExamplePluginResponse"}));
        return true;
    }
    return false;
};
/**
 * This method handles outbound messages going to the client
 * @param message {object} The outgoing message to the client.
 * @param connection {WebSocketConnection} The web socket connection to the client.
 * @param state {DeviceState} The state of the device the client is connected to, false if no device leased.
 * @param lease {DeviceLease} Information about the device the client has leased, false if no device leased.
 * @param simulateInput {function} Function that allows you to simulate incoming messages from the client.
 * @return {boolean} True if the outbound message has been handled and should be ignored, false otherwise.
 */
ExamplePlugin.prototype.handleOutboundMessage  = function(message, connection, state, lease, simulateInput) {
    return false;
};

//This exposes the plugin to OmniScript.
var example = new ExamplePlugin();
module.exports = example;