function CommandLine() {
    "use strict";
    this.args = {};
}

// override the settings in config with any setting passed in on the command line (after parsing)
CommandLine.prototype.override = function (config) {
    var webServer = Object.assign(config.webServer, this.args.webServer);
    var discovery = Object.assign(config.discovery, this.args.discovery);
    config.webServer = webServer;
    config.discovery = discovery;
}

// parse the command line arguments.
CommandLine.prototype.parse = function(argv) {
    "use strict";
    
    var ss; // temporary string manipulation
    this.args = {
        "webServer": {},
        "discovery": {}
    };
    var i = 2;
    while (i < argv.length) {

        if (argv[i].startsWith("--discoveryPort=")) {
            ss = argv[i].split("=")[1];
            if (parseInt(ss) != ss) {
                throw ("Error in argument " + argv[i] + "', port is not an integer.");
            }
            this.args.discovery.port = parseInt(ss);
        }
        else if (argv[i] == "-dp") {
            i++;
            if (parseInt(argv[i]) != argv[i]) {
                throw ("Error in argument '-dp " + argv[i] + "', port is not an integer.");
            }
            this.args.discovery.port = parseInt(argv[i]);
        }
        else if (argv[i].startsWith("--discoveryListenAddress=")) {
            ss = argv[i].split("=")[1];
            if ((parseInt(ss) != ss) && (ss !== "*") && (ss !== "localhost")) {
                throw ("Error in argument '--discoveryListenAddress=" + argv[i] + "', port is not an integer or * or 'localhost'.");
            }
            var n = parseInt(ss);
            if (Number.isInteger(n)) {
                ss = n;
            }
            this.args.discovery.listenAddress = ss;
        }
        else if (argv[i] == "-dl") {
            i++;
            if ((parseInt(argv[i]) != argv[i]) && (argv[i] !== "*") && (argv[i] !== "localhost")) {
                throw ("Error in argument '-dl " + argv[i] + "', port is not an integer or * or 'localhost'.");
            }
            var n = parseInt(argv[i]);
            if (Number.isInteger(n)) {
                ss = n;
            } else {
                ss = argv[i];
            }
            this.args.discovery.listenAddress = ss;
        }
        else if (argv[i].startsWith("--webserverListenAddress=")) {
            ss = argv[i].split("=")[1];
            if ((parseInt(ss) != ss) && (ss !== "*") && (ss !== "localhost")) {
                throw ("Error in argument '--webserverListenAddress=" + argv[i] + "', port is not an integer or * or 'localhost'.");
            }
            var n = parseInt(ss);
            if (Number.isInteger(n)) {
                ss = n;
            }
            this.args.webServer.listenAddress = ss;
        }
        else if (argv[i] == "-wl") {
            i++;
            if ((parseInt(argv[i]) != argv[i]) && (argv[i] !== "*") && (argv[i] !== "localhost")) {
                throw ("Error in argument '-wl " + argv[i] + "', port is not an integer or * or 'localhost'.");
            }
            var n = parseInt(argv[i]);
            if (Number.isInteger(n)) {
                ss = n;
            } else {
                ss = argv[i];
            }
            this.args.webServer.listenAddress = ss;
        }
        else if (argv[i] == "-df") {
            this.args.discovery.discoveryOn = false;
        }
        else if (argv[i].startsWith("--http=")) {
            ss = argv[i].split("=")[1];
            if ((ss != 'true') && (ss != 'false')) {
                throw ("Error --http must be either true or false");
            }
            this.args.webServer.http = (ss == 'true');
        }
        else if (argv[i] == "-pt") {
            this.args.webServer.http = true;
        }
        else if (argv[i] == "-pf") {
            this.args.webServer.http = false;
        }
        else if (argv[i].startsWith("--https=")) {
            ss = argv[i].split("=")[1];
            if ((ss != 'true') && (ss != 'false')) {
                throw ("Error --https must be either true or false");
            }
            this.args.webServer.https = (ss == 'true');
        }
        else if (argv[i] == "-st") {
            this.args.webServer.https = true;
        }
        else if (argv[i] == "-sf") {
            this.args.webServer.https = false;
        }
        else if (argv[i].startsWith("--httpPort=")) {
            ss = argv[i].split("=");
            if ((ss.length != 2) || (parseInt(ss[1]) != ss[1])) {
                throw ("Error in argument '" + argv[i] + "', port is not an integer.");
            }
            this.args.webServer.httpPort = parseInt(ss[1]);
        }
        else if (argv[i] == "-p") {
            i++;
            if (parseInt(argv[i]) != argv[i]) {
                throw ("Error in argument '-p " + argv[i] + "', port is not an integer.");
            }
            this.args.webServer.httpPort = parseInt(argv[i]);
        }
        else if (argv[i].startsWith("--httpsPort=")) {
            ss = argv[i].split("=");
            if ((ss.length != 2) || (parseInt(ss[1]) != ss[1])) {
                throw ("Error in argument '" + argv[i] + "', port is not an integer.");
            }
            this.args.webServer.httpsPort = parseInt(ss[1]);
        }
        else if (argv[i] == "-s") {
            i++;
            if (parseInt(argv[i]) != argv[i]) {
                throw ("Error in argument '-s " + argv[i] + "', port is not an integer.");
            }
            this.args.webServer.httpsPort = parseInt(argv[i]);
        }
        else if ((argv[i] == "--help") || (argv[i] == "-h") || (argv[i] == "-?")) {
            console.log("Command line argument options:");
            console.log("  --help | -? | -h, this help");
            console.log("  --discoveryPort=n, the UDP port to listen to for device connection messages");
            console.log("  --discoveryListenAddress=n|*|localhost, the UDP port that is accepted for listening");
            console.log("  --webserverListenAddress=n|*|localhost, the UDP port that is accepted for listening");
            console.log("  --http=true|false");
            console.log("  --https=true|false");
            console.log("  --httpPort=n, where n is a decimal port number");
            console.log("  --httpsPort=n, where n is a decimal port number");
            console.log("  -dp is the same as --discoveryPort=n");
            console.log("  -dl is the same as --discoveryListenAddress=n|*|localhost");
            console.log("  -wl is the same as --webserverListenAddress=n|*|localhost");
            console.log("  -pt is the same as --http=true");
            console.log("  -pf is the same as --http=false");
            console.log("  -st is the same as --https=true");
            console.log("  -sf is the same as --https=false");
            console.log("  -p n, is the same as --httpPort=n, where n is a decimal port number");
            console.log("  -s n, is the same as --httpsPort=n, where n is a decimal port number");
            console.log("These values override values found in scriptel-config.json");
            console.log("End of help");
            process.exit(203);
        }
        else {
            throw ("Error, unknown argument '" + argv[i] + "'");
        }
        i++;
    }
    return this.args;
}


var commandLine = new CommandLine();
//commandLine.parse(process.argv);
//console.log(JSON.stringify(commandLine.args));
module.exports = commandLine;