function defaultAppender(message) {
    "use strict";
    console.log(message);
}
function defaultFormatter(level, date, message, objects, ref) {
    "use strict";
    var x;
    if (typeof objects === 'object') {
        for (x in objects) {
			if (objects.hasOwnProperty(x)) {
                if (typeof objects[x] === "object" && objects[x].stack) {
                    //This is an Error
                    message = message.replace("{" + x + "}", objects[x].stack);
                } else {
                    message = message.replace("{" + x + "}", objects[x]);
                }
			}
        }
    }

    var caller = ref.getCaller();
    var msg = date + " " + caller["function"] + "   \n" + level + ": " + message;
    return msg;
}
function Logger() {
    "use strict";
    this.levels = {"NONE": 200, "SEVERE": 100, "WARNING": 90, "INFO": 80, "CONFIG": 70, "FINE": 60, "FINER": 50, "FINEST": 40, "ALL": 0};
    this.appenders = [defaultAppender];
    this.formatter = defaultFormatter;
    this.level = this.levels.ALL;
    this.config = false;
}
Logger.prototype.setConfiguration = function (config) {
    "use strict";
    this.config = config;
};
Logger.prototype.log = function (level, message, objects) {
    "use strict";
    var x;
    if (level > this.level) {
        var msg = this.formatter(this.getLevel(level), new Date(), message, objects, this);
        for (x = 0; x < this.appenders.length; x++) {
            this.appenders[x](msg);
        }
    }
};
Logger.prototype.severe = function (message, objects) {
    "use strict";
    this.log(this.levels.SEVERE, message, objects);
};
Logger.prototype.warning = function (message, objects) {
    "use strict";
    this.log(this.levels.WARNING, message, objects);
};
Logger.prototype.info = function (message, objects) {
    "use strict";
    this.log(this.levels.INFO, message, objects);
};
Logger.prototype.config = function (message, objects) {
    "use strict";
    this.log(this.levels.CONFIG, message, objects);
};
Logger.prototype.fine = function (message, objects) {
    "use strict";
    this.log(this.levels.FINE, message, objects);
};
Logger.prototype.finer = function (message, objects) {
    "use strict";
    this.log(this.levels.FINER, message, objects);
};
Logger.prototype.finest = function (message, objects) {
    "use strict";
    this.log(this.levels.FINEST, message, objects);
};
Logger.prototype.getLevel = function (level) {
    "use strict";
    var l;
    if (parseInt(level, 10) === level) {
        for (l in this.levels) {
            if (this.levels.hasOwnProperty(l)) {
                if (level === this.levels[l]) {
                    return l;
                }
            }
        }
    } else if ((level.toString()) === level) {
        return this.levels[level.toUppwerCase()];
    }
    return undefined;
};


Logger.prototype.getCaller = function () {
    "use strict";
    var x;
    var stack = new Error().stack.split('\n');
    var regex = /\s*at (.*) \((.*):(\d*):(\d*)\)/;
    for (x = 0; x < stack.length; x++) {
        var line = stack[x];
        if (line.indexOf("Logger.") < 0) {
            var data = line.match(regex);
            if (data) {
                return {"function": data[1], "file": data[2], "line": data[3], "character": data[4]};
            }
        }
    }
    return false;
};
var logger = new Logger();
module.exports = logger;