var fs = require('fs');
/**
 * This class is responsible for loading plugins and passing
 * the data to/from the plugins.
 */
function Plugins() {
    this.plugins = [];
    this.config = undefined;
    this.logger = undefined;
}
Plugins.prototype.setConfiguration = function(config) {
    this.config = config;
};
Plugins.prototype.setLogger = function(logger) {
    this.logger = logger;
};
Plugins.prototype.load = function() {
    var t = this;
    fs.readdir(this.config.plugins.path,function(err,files) {
        var x;
        for(x=0;x<files.length;x++) {
            var file = files[x];
            var idx = file.lastIndexOf(".");
            if(idx>=0) {
                var ext = file.substring(idx).toLowerCase();
                if(ext === ".js" && file.substring(0,7).toLowerCase()==="plugin-") {
                    //We've found a viable plugin.
                    t.loadPlugin(file);
                };
            }
        }
    });
};
Plugins.prototype.loadPlugin = function(file) {
    var path = "../"+this.config.plugins.path+"/"+file;
    var plugin = require(path);
    this.logger.info("Successfully loaded "+plugin.name+" ("+plugin.version+")");
    this.plugins[this.plugins.length] = plugin;
};
Plugins.prototype.handleOutboundMessage = function(message, connection, state, lease, simulateInput, sendOutput) {
    var handled = false;
    var x=0;
    for(x=0;!handled && x<this.plugins.length;x++) {
        var plugin = this.plugins[x];
        if(plugin.handleOutboundMessage(message, connection, state, lease, simulateInput, sendOutput)) {
            handled = true;
        }
    }
    return handled;
};
Plugins.prototype.handleInboundMessage = function(message, connection, state, lease, simulateInput, sendOutput) {
    var handled = false;
    var x=0;
    for(x=0;!handled && x<this.plugins.length;x++) {
        var plugin = this.plugins[x];
        if(plugin.handleInboundMessage(message, connection, state, lease, simulateInput, sendOutput)) {
            handled = true;
        }
    }
    return handled;
};

var plugins = new Plugins();
module.exports = plugins;