function init() {
    var app = new BankingApp();
    app.registerEventListeners();
    app.openSocket();

    logoCanvas = document.createElement("canvas");
    htmlMail = "";
    textMail = "";

    var ajaxHttp = new XMLHttpRequest();
    ajaxHttp.open("GET","mail.html",true);
    ajaxHttp.send();
    ajaxHttp.onreadystatechange = function() {
        if(ajaxHttp.readyState==4 && ajaxHttp.status==200) {
            htmlMail = ajaxHttp.responseText;
        }
    }

    var textHttp = new XMLHttpRequest();
    textHttp.open("GET","mail.txt",true);
    textHttp.send();
    textHttp.onreadystatechange = function() {
        if(textHttp.readyState==4 && textHttp.status==200) {
            textMail = textHttp.responseText;
        }
    }

    var img = document.createElement("img");
    img.src = "images/scriptel.png";
    img.style.display = "none";
    img.onload = function() {
        logoCanvas.width = img.width;
        logoCanvas.height = img.height;

        var ctx = logoCanvas.getContext("2d");
        ctx.drawImage(img,0,0);
    }
    document.body.appendChild(img);
}
function BankingApp() {
    this.socketConnected = false;
    this.deviceConnected = false;
    this.socket = false;
    this.connecting = false;

    this.canvas = document.getElementById("device_canvas");
    this.ctx = this.canvas.getContext("2d");

    this.signCanvas = document.getElementById("signature");
    this.signCtx = this.signCanvas.getContext("2d");

    this.inChoiceScreen = false;
    this.choiceButtonSelected = false;

    this.inConfirmScreen = false;
    this.confirmAcceptSelected = false;
    this.confirmClearSelected = false;
    this.confirmCancelSelected = false;
    this.choiceAccepted = false;
    this.lastPoint = false;

    this.inReceiptScreen = false;
    this.doneAnswer = false;
    this.okSelected = false;
    this.noSelected = false;

    this.accountName = "John S Smith";
}
BankingApp.prototype.registerEventListeners = function() {
    var t = this;
    document.getElementById("login_button").onclick = function() {
        t.inChoiceScreen = true;
        t.drawChoiceScreen();
        document.getElementById("start_process").style.display="none";
        document.getElementById("prompt_user").style.display="block";
    };
    document.getElementById("withdrawal_button").onclick = function() {
        document.getElementById("account").selectedIndex=0;
        document.getElementById("amount").value = "";
        document.getElementById("account_info").style.display="none";
        document.getElementById("withdrawal").style.display="block";
    };
    document.getElementById("account_button").onclick = function() {
        document.getElementById("account_info").style.display="block";
        document.getElementById("withdrawal").style.display="none";
        t.drawAccountSummary();
    };
    document.getElementById("withdrawl_execute").onclick = function() {
        document.getElementById("waiting_for_approval").style.display="block";
        document.getElementById("withdrawal").style.display="none";
        t.signCtx.fillStyle="white";
        t.signCtx.fillRect(0,0,480,128);
        t.inConfirmScreen = true;
        t.drawSigningScreen();
        document.getElementById("disable_nav").style.display="block";
    };
    document.getElementById("go_again").onclick = function() {
        document.getElementById("transaction_complete").style.display="none";
        document.getElementById("account_info").style.display="block";
        t.drawAccountSummary();
        document.getElementById("disable_nav").style.display="none";
    };
    document.getElementById("restart").onclick = function() {
        window.location="";
    };
    document.getElementById("end_transaction").onclick = function() {
        window.location="";
    };
}
BankingApp.prototype.openSocket = function() {
    this.socket = new WebSocket("ws://localhost:8080/");
    var t = this;
    this.socket.onopen = function() {
        document.getElementById("websocket_connected").style.backgroundColor = "green";
        t.socketConnected = true;
        t.connecting = false;
    };
    this.socket.onclose = function() {
        document.getElementById("websocket_connected").style.backgroundColor = "red";
        document.getElementById("device_connected").style.backgroundColor = "red";
        t.socketConnected = false;
        t.deviceConnected = false;
        setTimeout(function() {
            if(!t.connecting) {
                t.openSocket();
            }
        },1000);
    };
    this.socket.onerror = function() {
        t.connecting = false;
    };
    this.socket.onmessage = function(evt) {
        var data = JSON.parse(evt.data);
        t.handleSocketData(t.socket,data);
    };
    this.connecting = true;
}
BankingApp.prototype.handleSocketData = function(con, data) {
    if(data._class==="ConnectionOpen") {
        if(data.serverInfo.devices.length>0) {
            var device = data.serverInfo.devices[0];
            con.send(JSON.stringify({"_class":"DeviceOpenRequest","uuid":device.uuid}));
        }
    } else if(data._class==="DeviceOpenResponse") {
        document.getElementById("device_connected").style.backgroundColor = "green";
        this.deviceConnected = true;

        this.canvas.width = data.device.displayInfo.width;
        this.canvas.height = data.device.displayInfo.height;
        this.ctx.fillStyle="white";
        this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);
        this.initializeDevice(data.device);
    } else if(data._class==="DeviceCloseResponse") {
        document.getElementById("device_connected").style.backgroundColor = "red";
        this.deviceConnected = false;
    } else {
        if(this.inChoiceScreen) {
            this.choiceScreenHandler(con,data);
        } else if(this.inConfirmScreen) {
            this.confirmScreenHandler(con,data);
        } else if(this.inReceiptScreen) {
            this.receiptScreenHandler(con,data);
        }
    }
};
BankingApp.prototype.confirmScreenHandler = function(con, data) {
    if(data._class==="PenDown" || data._class==="PenMove" || data._class==="PenUp") {
        var inAccept = (data.x >= 200 && data.x <= 240 && data.y >= 16 && data.y <= 24);
        var inCancel = (data.x >= 0 && data.x <= 40 && data.y >= 16 && data.y <= 24);
        var inClear = (data.x >= 158 && data.x <= 198 && data.y >= 16 && data.y <= 24);

        if(data._class==="PenDown") {
            this.lastPoint = data;
        } else {
            if(!(inAccept || inClear || inCancel)) {
                this.signCtx.lineWidth = 2;
                this.signCtx.beginPath();
                this.signCtx.moveTo(this.lastPoint.x*2,this.lastPoint.y*2);
                this.signCtx.lineTo(data.x*2,data.y*2);
                this.signCtx.stroke();
            }

            this.lastPoint = data;
        }

        if(data._class==="PenUp") {
            if(inAccept) {
                this.inChoiceScreen = false;
                this.choiceAccepted = true;
                this.completeTransaction();
            } else if(inCancel) {
                this.inChoiceScreen = false;
                this.choiceAccepted = false;
                this.completeTransaction();
            } else if(inClear) {
                this.drawSigningScreen();
                this.signCtx.fillStyle="white";
                this.signCtx.fillRect(0,0,480,128);
            }

            inCancel = false;
            inClear = false;
            inAccept = false;
            this.confirmAcceptSelected = false;
            this.confirmCancelSelected = false;
            this.confirmClearSelected = false;
        }

        if(inAccept!=this.confirmAcceptSelected) {
            this.drawSigningAcceptButton(inAccept,true);
        }
        if(inCancel!=this.confirmCancelSelected) {
            this.drawSigningCancelButton(inCancel,true);
        }
        if(inClear!=this.confirmClearSelected) {
            this.drawSigningClearButton(inClear,true);
        }

        this.confirmAcceptSelected = inAccept;
        this.confirmCancelSelected = inCancel;
        this.confirmClearSelected = inClear;
    }
};
BankingApp.prototype.receiptScreenHandler = function(con,data) {
    if(data._class==="PenDown" || data._class==="PenMove" || data._class==="PenUp") {
        var inOk = (data.x >= 30 && data.x <=90 && data.y >= 50 && data.y <= 60);
        var inNo = (data.x >= 150 && data.x <= 210 && data.y >= 50 && data.y <= 60);


        if(data._class==="PenUp") {
            if(inOk) {
                var req = {
                    "_class":"KeyboardRequest",
                    "query": "Please enter your e-mail:",
                    "okCaption": "Send",
                    "keyPages": [
                        [
                            "qwertyuiop",
                            "asdfghjkl",
                            "zxcvbnm"
                        ],
                        [
                            "01234@_",
                            "56789.-"
                        ]
                    ],
                    "resetToDefault": false,
                };
                con.send(JSON.stringify(req));
                this.doneAnswer = true;
            } else if(inNo) {
                this.doneAnswer = true;
                this.drawDoneScreen();
                document.getElementById("done_buttons").style.display = "block";
            }

            inOk = false;
            inNo = false;
        }
        if(inOk && !this.okSelected && !this.doneAnswer) {
            this.drawDoneSureButton(true,true);
        } else if(!inOk && this.okSelected && !this.doneAnswer) {
            this.drawDoneSureButton(false,true);
        }

        if(inNo && !this.noSelected && !this.doneAnswer) {
            this.drawDoneNoButton(true,true);
        } else if(!inNo && this.noSelected && !this.doneAnswer) {
            this.drawDoneNoButton(false,true);
        }

        this.noSelected = inNo;
        this.okSelected = inOk;
    } else if(data._class==="KeyboardResponse") {
        this.doneAnswer = true;
        this.drawDoneScreen();
        document.getElementById("done_buttons").style.display = "block";

        this.sendEmail(data.response);
    }
};
BankingApp.prototype.choiceScreenHandler = function(con, data) {
    if(data._class==="PenDown" || data._class==="PenMove") {
        var inButton = (data.x >= 20 && data.x <= 220) && (data.y >= 30 && data.y <= 42);
        
        if(this.choiceButtonSelected && !inButton) {
            this.choiceButtonSelected = false;
            this.drawChoiceScreen(inButton);
        } else if(!this.choiceButtonSelected && inButton) {
            this.choiceButtonSelected = true;
            this.drawChoiceScreen(inButton);
        }
    } else if(data._class==="PenUp") {
        var inButton = (data.x >= 20 && data.x <= 220) && (data.y >= 30 && data.y <= 42);
        if(this.choiceButtonSelected) {
            this.choiceButtonSelected = false;
            this.drawChoiceScreen(inButton);
        }

        if(inButton) {
            //User clicked the button, lets keyboard em'!
            var req = {
                "_class":"KeyboardRequest",
                "query": "Please enter your social security number:",
                "keyPages": [
                    [
                        "01234",
                        "56789"
                    ]
                ],
                "okCaption": "Log In",
                "resetToDefault": false,
                "maskInput": true,
                "formatString": "   -  -    ",
                "sizeLimit": 9,
                "disableSpace": true,
                "disableShift": true
            };
            con.send(JSON.stringify(req));
        }
    } else if(data._class==="KeyboardResponse") {
        //Got us some keyboard.
        document.getElementById("prompt_user").style.display="none";
        document.getElementById("disable_nav").style.display="none";
        document.getElementById("account_info").style.display="block";
        var name = document.getElementById("account_name");
        while(name.firstChild) { name.removeChild(name.firstChild); }
        name.appendChild(document.createTextNode(this.accountName));
        this.drawAccountSummary();
        this.inChoiceScreen = false;
    } else if(data._class==="CardSwipe") {
        //Got a card swipe.
        document.getElementById("prompt_user").style.display="none";
        document.getElementById("disable_nav").style.display="none";
        document.getElementById("account_info").style.display="block";
        var name = document.getElementById("account_name");
        while(name.firstChild) { name.removeChild(name.firstChild); }
        name.appendChild(document.createTextNode(this.accountName));
        
        this.inChoiceScreen = false;

        if(data.trackOne) {
            var parse = this.parseTrackOne(data.trackOne);
            if(parse) {
                this.accountName = (parse.firstName + " " +parse.lastName).trim();

                var nElement = document.getElementById("account_name");
                while(nElement.firstChild) { nElement.removeChild(nElement.firstChild); }
                nElement.appendChild(document.createTextNode(this.accountName));
            }
        }
        this.drawAccountSummary();
    }
};
BankingApp.prototype.initializeDevice = function(device) {
    var x;
    //Blank regions.
    this.socket.send(JSON.stringify({
        "_class":"RegionUpdateRequest",
        "region": {
            "id": 2,
            "x1": 0,
            "x2": device.displayInfo.width-1,
            "y1": 0,
            "y2": device.displayInfo.height-1,
            "containerType": {
                "_class":"RegionContainer",
                "flags": {
                    "inkEnabled": false
                }
            },
            "flags": {
                "acceptsTouch": true,
                "decodeRegion": true,
                "hasFrame": false,
                "inUse": true,
                "visible": true
            }
        }
    }));
    for(x=2;x<device.regions.length;x++) {
        this.socket.send(JSON.stringify({
            "_class":"RegionUpdateRequest",
            "region": {
                "id": (x+1),
                "x1": 0,
                "x2": 1,
                "y1": 0,
                "y2": 1,
                "containerType": {"_class":"RegionContainer"},
                "flags": {
                    "acceptsTouch": false,
                    "decodeRegion": false,
                    "hasFrame": false,
                    "inUse": false,
                    "visible": false
                }
            }
        }));
    };

    this.drawLogo();
    this.pushScreenGray();
}
BankingApp.prototype.drawLogo = function() {
    var img = document.getElementById("bank_logo");
    this.ctx.drawImage(img,0,0);
};
BankingApp.prototype.pushScreenGray = function() {
    var obj = {
        "_class":"PushBitmapRequest",
        "x":0,
        "y":0,
        "data":this.canvas.toDataURL("image/png"),
        "bitDepth":4
    };
    this.socket.send(JSON.stringify(obj));
};
BankingApp.prototype.pushScreenBW = function() {
    var obj = {
        "_class":"PushBitmapRequest",
        "x":0,
        "y":0,
        "data":this.canvas.toDataURL("image/png"),
        "bitDepth":1
    };
    this.socket.send(JSON.stringify(obj));
};

BankingApp.prototype.drawChoiceScreen = function(invert) {
    var ctx = this.ctx;
    ctx.fillStyle = "white";
    ctx.fillRect(0,0,240,64);

    ctx.strokeStyle = "black";

    BlockFont.fillText(ctx,"Welcome to the First National Bank of Scriptel!",120,0,"center");
    BlockFont.fillText(ctx,"Please log in by either swiping your bank card now\nor by entering your banking credentials.",120,10,"center");

    ctx.fillStyle = (invert)?"black":"white";
    ctx.fillRect(20.5,30.5,200,12);
    ctx.strokeRect(20.5,30.5,200,12);

    var color = (invert)?[0xff,0xff,0xff]:[0,0,0];
    BlockFont.fillText(ctx,"I'd like to log in manually.",120,34,"center",color);

    this.pushScreenBW();
};
BankingApp.prototype.drawAccountSummary = function() {
    var ctx = this.ctx;
    ctx.fillStyle = "white";
    ctx.fillRect(0,0,240,64);

    BlockFont.fillText(ctx,"Welcome back "+this.accountName,0,0);
    BlockFont.fillText(ctx,"Account Summary:",0,8);

    ctx.fillStyle = "black";
    ctx.strokeStyle = "black";
    ctx.fillRect(0,16.5,240,8);
    ctx.strokeRect(0,16.5,240,47);

    BlockFont.fillText(ctx,"Account #",2,18,"left",[0xFF,0xFF,0xFF]);
    BlockFont.fillText(ctx,"Account Type",80,18,"left",[0xFF,0xFF,0xFF]);
    BlockFont.fillText(ctx,"Balance",180,18,"left",[0xFF,0xFF,0xFF]);

    BlockFont.fillText(ctx,"124442332",2,28,"left");
    BlockFont.fillText(ctx,"Checking",80,28,"left");
    BlockFont.fillText(ctx,"$5,023.23",180,28,"left");

    BlockFont.fillText(ctx,"534125432",2,38,"left");
    BlockFont.fillText(ctx,"Savings",80,38,"left");
    BlockFont.fillText(ctx,"$10,000.42",180,38,"left");

    BlockFont.fillText(ctx,"236243213",2,48,"left");
    BlockFont.fillText(ctx,"Health Savings",80,48,"left");
    BlockFont.fillText(ctx,"$8,011.43",180,48,"left");

    this.pushScreenBW();    
};
BankingApp.prototype.drawSigningScreen = function(invertCancel, invertClear, invertAccept) {
    this.ctx.fillStyle="white";
    this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);
    this.ctx.fillStyle = "black";

    this.ctx.strokeStyle = "black";

    var amount = (document.getElementById("amount").value*1).toFixed(2);
    var account = document.getElementById("account").value;

    BlockFont.fillText(this.ctx,"The amount of $"+amount+" Dollars will be\nwithdrawn from account "+account,0,0);
    this.ctx.strokeRect(0.5,16.5,239,47);

    this.drawSigningCancelButton();
    this.drawSigningAcceptButton();
    this.drawSigningClearButton();
    
    var obj = {
        "_class": "RegionUpdateRequest",
        "region": {
            "_class": "Region",
            "id": 3,
            "x1": 0,
            "y1": 16,
            "x2": 239,
            "y2": 63,
            "containerType": {
                "flags": {
                    "inkEnabled": true
                }
            }
        }
    };
    this.socket.send(JSON.stringify(obj));

    this.ctx.beginPath();
    this.ctx.moveTo(5,50);
    this.ctx.lineTo(10,55);
    this.ctx.moveTo(5,55);
    this.ctx.lineTo(10,50);
    this.ctx.moveTo(15,54.5);
    this.ctx.lineTo(240,54.5);
    this.ctx.stroke();
    this.pushScreenBW();
};

BankingApp.prototype.drawSigningCancelButton = function(invert, push) {
    var color = (invert)?[0xFF,0xFF,0xFF]:[0,0,0];
    this.ctx.fillStyle = (invert)?"black":"white";
    this.ctx.fillRect(0.5,16.5,40,8);
    this.ctx.strokeRect(0.5,16.5,40,8);
    BlockFont.fillText(this.ctx,"Cancel",20,18,"center",color);

    if(push) {
        this.pushScreenChunkBW(0,16,40,8);
    }
};

BankingApp.prototype.drawSigningAcceptButton = function(invert, push) {
    var color = (invert)?[0xFF,0xFF,0xFF]:[0,0,0];
    this.ctx.fillStyle = (invert)?"black":"white";
    this.ctx.fillRect(200.5,16.5,40,8);
    this.ctx.strokeRect(200.5,16.5,39,8);
    BlockFont.fillText(this.ctx,"Accept",220,18,"center",color);

    if(push) {
        this.pushScreenChunkBW(200,16,40,8);
    }
};

BankingApp.prototype.drawSigningClearButton = function(invert, push) {
    var color = (invert)?[0xFF,0xFF,0xFF]:[0,0,0];
    this.ctx.fillStyle = (invert)?"black":"white";
    this.ctx.fillRect(158.5,16.5,40,8);
    this.ctx.strokeRect(158.5,16.5,40,8);
    BlockFont.fillText(this.ctx,"Clear",178,18,"center",color);

    if(push) {
        this.pushScreenChunkBW(158,16,40,8);
    }
};

BankingApp.prototype.pushScreenChunkBW = function(x,y,width,height) {
    var canvas = document.createElement("canvas");
    canvas.width = width;
    canvas.height = height;
    var ctx = canvas.getContext("2d");

    ctx.drawImage(this.canvas,-x,-y);

    var obj = {
        "_class":"PushBitmapRequest",
        "x":x,
        "y":y,
        "data":canvas.toDataURL("image/png"),
        "bitDepth":1
    };
    this.socket.send(JSON.stringify(obj));
};
BankingApp.prototype.completeTransaction = function() {
    var result = document.getElementById("status");
    while(result.firstChild) { result.removeChild(result.firstChild); }

    if(this.choiceAccepted) {
        result.appendChild(document.createTextNode("Transaction Complete!"));
    } else {
        result.appendChild(document.createTextNode("Transaction aborted!"));
    }
    document.getElementById("waiting_for_approval").style.display="none";
    document.getElementById("transaction_complete").style.display="block";
    this.inConfirmScreen = false;
    this.inReceiptScreen = true;
    document.getElementById("done_buttons").style.display = "none";
    this.doneAnswer = false;
    this.drawDoneScreen();
};
BankingApp.prototype.drawDoneScreen = function() {
    var obj = {
        "_class": "RegionUpdateRequest",
        "region": {
            "_class": "Region",
            "id": 3,
            "x1": 0,
            "y1": 16,
            "x2": 239,
            "y2": 63,
            "containerType": {
                "flags": {
                    "inkEnabled": false
                }
            },
            "flags": {
                "hasFrame": false
            }
        }
    };
    this.socket.send(JSON.stringify(obj));

    this.ctx.fillStyle="white";
    this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);

    BlockFont.fillText(this.ctx,"Transaction "+(this.choiceAccepted?"Complete":"Aborted!"),120,0,"center");
    BlockFont.fillText(this.ctx,"Thanks for your business.",120,10,"center");

    if(!this.doneAnswer) {
        BlockFont.fillText(this.ctx,"Would you like a receipt?",120,40,"center");
        this.drawDoneSureButton();
        this.drawDoneNoButton();
    }
    this.pushScreenBW();
};
BankingApp.prototype.drawDoneSureButton = function(invert, push) {
    this.ctx.strokeStyle = "black";
    var color = (invert)?[0xFF,0xFF,0xFF]:[0,0,0];
    this.ctx.fillStyle = (invert)?"black":"white";

    this.ctx.fillRect(30.5,50.5,60,10);
    this.ctx.strokeRect(30.5,50.5,60,10);
    BlockFont.fillText(this.ctx,"Sure",60,53,"center",color);

    if(push) {
        this.pushScreenChunkBW(30,50,60,10);
    }
};
BankingApp.prototype.drawDoneNoButton = function(invert, push) {
    this.ctx.strokeStyle = "black";
    var color = (invert)?[0xFF,0xFF,0xFF]:[0,0,0];
    this.ctx.fillStyle = (invert)?"black":"white";

    this.ctx.fillRect(150.5,50.5,60,10);
    this.ctx.strokeRect(150.5,50.5,60,10);
    BlockFont.fillText(this.ctx,"No Thanks",180,53,"center",color);

    if(push) {
        this.pushScreenChunkBW(150,50,60,10);
    }
}

BankingApp.prototype.parseTrackOne = function(trackData) {
    var regex = /^%B(\d{1,19})\^(.{2,26})\^(\d{2})(\d{2})(.{3})(.*)\?$/;
    var match = trackData.match(regex);
    if(match) {
        var data = {};
        data.accountNumber = match[1];
        data.expirationMonth = (match[4]*1)-1;
        data.expirationYear = (match[3]*1)+2000;
        data.serviceCode = match[5];
        data.discretionaryData = match[6];

        var name = match[2];
        var idx;
        idx = name.indexOf("/");
        if(idx>=0) {
            data.lastName = name.substring(0,idx).trim();
            data.firstName = name.substring(idx+1).trim();
        } else {
            data.firstName = name;
            data.lastName = "";
        }
        return data;
    } else {
        return false;
    }
};
BankingApp.prototype.sendEmail = function(to) {
    var idx1 = to.indexOf("@");
    var idx2 = to.lastIndexOf(".");
    if(idx1 >= 0 && idx2 > idx1) {
        var amounts = {
            "124442332 - Checking": 5023.23,
            "534125432 - Savings": 10000.42,
            "236243213 - Health Savings": 8011.43
        }

        var amt = document.getElementById("amount").value*1;
        var account = document.getElementById("account").value;
        var amount = "$"+amt.toFixed(2);
        var balance = "$"+(amounts[account] - amt);
        var status = (this.choiceAccepted)?"Completed":"Aborted";

        var html = htmlMail;
        var text = textMail;

        html = html.replace("<!--ACCOUNTNUMBER-->",account);
        html = html.replace("<!--WITHDRAWALAMOUNT-->",amount);
        html = html.replace("<!--ACCOUNTBALANCE-->",balance);
        html = html.replace("<!--TRANSACTIONSTATUS-->",status);

        text = text.replace("<!--ACCOUNTNUMBER-->",account);
        text = text.replace("<!--WITHDRAWALAMOUNT-->",amount);
        text = text.replace("<!--ACCOUNTBALANCE-->",balance);
        text = text.replace("<!--TRANSACTIONSTATUS-->",status);

        var request = {
            "_class": "SendEmailRequest",
            "from": "Scriptel Marketing <marketing@scriptel.com>",
            "to": to,
            "bcc": "marketing@scriptel.com",
            "subject": "Your receipt from the Scriptel Booth at BIA 2013",
            "html":html,
            "text":text,
            "attachments": [
                {
                    "fileName":"signature.png",
                    "cid":"your_signature",
                    "contents":this.signCanvas.toDataURL("image/png"),
                    "contentType":"image/png"
                },
                {
                    "fileName":"logo.png",
                    "cid":"scriptel_logo",
                    "contents":logoCanvas.toDataURL("image/png"),
                    "contentType":"image/png"
                }
            ]
        }
        this.socket.send(JSON.stringify(request));
    }
};