function init() {
    cart = new ShoppingCart();
    cart.registerListeners();
    cart.openSocket();

    cart.testPayment();
}
function ShoppingCart() {
    this.socketConnected = false;
    this.deviceConnected = false;
    this.socket = false;
    this.connecting = false;

    this.canvas = document.getElementById("screen");
    this.ctx = this.canvas.getContext("2d");

    this.signCanvas = document.getElementById("agree_sign");
    this.signCtx = this.signCanvas.getContext("2d");
    this.signCtx.lineWidth = 2;
    this.signCtx.strokeStyle = "black";
    this.lastPoint = false;

    this.inReview = false;
    this.buttonDepressed = false;
    this.inSign = false;
}
ShoppingCart.prototype.openSocket = function() {
    this.socket = new WebSocket("ws://localhost:8080/");
    var t = this;
    this.socket.onopen = function() {
        document.getElementById("websocket_connected").style.backgroundColor = "green";
        t.socketConnected = true;
        t.connecting = false;
    };
    this.socket.onclose = function() {
        document.getElementById("websocket_connected").style.backgroundColor = "red";
        document.getElementById("device_connected").style.backgroundColor = "red";
        t.socketConnected = false;
        t.deviceConnected = false;
        setTimeout(function() {
            if(!t.connecting) {
                t.openSocket();
            }
        },1000);
    };
    this.socket.onerror = function() {
        t.connecting = false;
    };
    this.socket.onmessage = function(evt) {
        var data = JSON.parse(evt.data);
        t.handleSocketData(t.socket,data);
    };
    this.connecting = true;
}
ShoppingCart.prototype.registerListeners = function() {
    var t = this;
    document.getElementById("checkout_button").onclick = function() {
        document.getElementById("cart_contents").style.display = "none";
        document.getElementById("cart_payment").style.display = "block";
        t.drawScreen();
    };
    document.getElementById("billing_information").onchange = function(evt) {
        var nextButton = document.getElementById("continue_payment");
        if(document.getElementById("billing_information").checkValidity()) {
            nextButton.disabled = false;
        } else {
            nextButton.disabled = true;
        }
        t.drawScreen();
    };
    document.getElementById("shipping_information").onchange = function(evt) {
        var nextButton = document.getElementById("continue_shipping");
        if(document.getElementById("shipping_information").checkValidity()) {
            nextButton.disabled = false;
        } else {
            nextButton.disabled = true;
        }
        t.drawScreen();
    };
    document.getElementById("continue_payment").onclick = function() {
        document.getElementById("cart_payment").style.display = "none";
        document.getElementById("cart_shipping").style.display = "block";
        t.drawScreen();
    };
    document.getElementById("back_cart").onclick = function() {
        document.getElementById("cart_contents").style.display = "block";
        document.getElementById("cart_payment").style.display = "none";
        t.drawScreen();
    };
    document.getElementById("back_payment").onclick = function() {
        document.getElementById("cart_payment").style.display = "block";
        document.getElementById("cart_shipping").style.display = "none";
        t.drawScreen();
    };
    document.getElementById("continue_shipping").onclick = function() {
        var billingAddress1 = document.getElementById("billing_address_1").value;
        var billingAddress2 = document.getElementById("billing_address_2").value;
        var billingCity = document.getElementById("billing_city").value;
        var billingState = document.getElementById("billing_state").value;
        var billingZip = document.getElementById("billing_zip_code").value;

        var shippingAddress1 = document.getElementById("shipping_address_1").value;
        var shippingAddress2 = document.getElementById("shipping_address_2").value;
        var shippingCity = document.getElementById("shipping_city").value;
        var shippingState = document.getElementById("shipping_state").value;
        var shippingZip = document.getElementById("shipping_zip_code").value;
        var shippingInstructions = document.getElementById("shipping_instructions").value;

        var cardNumber = document.getElementById("billing_card_number").value;
        var nameOnCard = document.getElementById("billing_name_on_card").value;
        var cardExpirationMonth = document.getElementById("billing_expiration_month").value;
        var cardExpirationYear = document.getElementById("billing_expiration_year").value;

        var billingAddr = document.getElementById("review_billing_address");
        var shippingAddr = document.getElementById("review_shipping_address");

        var reviewName = document.getElementById("review_name_on_card");
        t.emptyElement(reviewName);
        reviewName.appendChild(document.createTextNode(nameOnCard));

        var reviewCard = document.getElementById("review_credit_card");
        t.emptyElement(reviewCard);
        var cardStr = t.checkCardType(cardNumber)+" ending in "+cardNumber.substring(cardNumber.length-4);
        reviewCard.appendChild(document.createTextNode(cardStr));

        var reviewSpecial = document.getElementById("review_special_instructions");
        t.emptyElement(reviewSpecial);
        reviewSpecial.appendChild(document.createTextNode(shippingInstructions));


        t.emptyElement(billingAddr);
        billingAddr.appendChild(t.formatAddress(billingAddress1,billingAddress2,billingCity,billingState,billingZip));

        t.emptyElement(shippingAddr);
        shippingAddr.appendChild(t.formatAddress(shippingAddress1,shippingAddress2,shippingCity,shippingState,shippingZip));

        document.getElementById("cart_shipping").style.display = "none";
        document.getElementById("cart_review").style.display = "block";
        t.inReview = true;
        t.drawScreen();
    };
    document.getElementById("back_shipping").onclick = function() {
        document.getElementById("cart_shipping").style.display = "block";
        document.getElementById("cart_review").style.display = "none";
        t.drawScreen();
        t.inReview = false;
    };
    document.getElementById("billing_card_number").onchange = function() {
        var number = document.getElementById("billing_card_number").value;
        
        var visa = document.getElementById("visa_icon");
        var masterCard = document.getElementById("mastercard_icon");
        var amex = document.getElementById("amex_icon");
        var discover = document.getElementById("discover_icon");
        var error = document.getElementById("card_validation_error");

        visa.className = "card_image card_image_disabled";
        masterCard.className = "card_image card_image_disabled";
        amex.className = "card_image card_image_disabled";
        discover.className = "card_image card_image_disabled";

        while(error.firstChild) { error.removeChild(error.firstChild); }

        var type = t.checkCardType(number);
        if(type) {
            if(type==="Visa") {
                visa.className = "card_image";
            } else if(type==="American Express") {
                amex.className = "card_image";
            } else if(type==="Discover") {
                discover.className = "card_image";
            } else if(type==="Master Card") {
                masterCard.className = "card_image";
            } else {
                error.appendChild(document.createTextNode("Sorry, we don't accept "+type+" cards."));
            }
        } else {
            if(number) {
                error.appendChild(document.createTextNode("Sorry, we don't accept your type of card."));
            }
        }
    };
    document.getElementById("shipping_copy_from_billing").onclick = function() {
        document.getElementById("shipping_address_1").value = document.getElementById("billing_address_1").value;
        document.getElementById("shipping_address_2").value = document.getElementById("billing_address_2").value;
        document.getElementById("shipping_city").value = document.getElementById("billing_city").value;
        document.getElementById("shipping_state").value = document.getElementById("billing_state").value;
        document.getElementById("shipping_zip_code").value = document.getElementById("billing_zip_code").value;
        document.getElementById("shipping_information").onchange();
    };
    document.getElementById("agree_to_terms").onchange = function() {
        if(this.checked) {
            document.getElementById("accept_pay").disabled = false;
        } else {
            document.getElementById("accept_pay").disabled = true;
        }
    };
    document.getElementById("accept_pay").onclick = function() {
        document.getElementById("cart_review").style.display = "none";
        document.getElementById("cart_thanks").style.display = "block";
        t.inReview = false;
        t.drawScreen();
    }
};

ShoppingCart.prototype.checkCardType = function(cardNumber) {
    var cardTypes = {
        "American Express":/^(34|37)\d{13}$/,
        "Diners Club":/^(30[0-5]\d{11}|36\d{12})$/,
        "Carte Blanche":/^38\d{12}$/,
        "Discover":/^6011\d{12}$/,
        "EnRoute":/^(2131|1800)\d{11}$/,
        "JCB":/^(3\d{15}|(2131|1800)\d{11})$/,
        "Master Card":/^5[1-5]\d{14}$/,
        "Visa":/^4\d{12,15}$/
    };

    var type;
    for(type in cardTypes) {
        var regex = cardTypes[type];
        if(cardNumber.match(regex)) {
            return type;
        }
    }
    return false;
};

ShoppingCart.prototype.testPayment = function() {
    document.getElementById("billing_card_number").value = "4000000000000";
    document.getElementById("billing_card_verify").value = "123";
    document.getElementById("billing_name_on_card").value = "John Smith";

    document.getElementById("billing_expiration_month").value = 7;
    document.getElementById("billing_expiration_year").value = "2014";
    document.getElementById("billing_address_1").value = "2007 State Route 104";
    document.getElementById("billing_city").value = "Ontario";
    document.getElementById("billing_state").value = "NY";
    document.getElementById("billing_zip_code").value = "14519";

    document.getElementById("shipping_address_1").value = "2007 State Route 104";
    document.getElementById("shipping_address_2").value = "";
    document.getElementById("shipping_city").value = "Ontario";
    document.getElementById("shipping_state").value = "NY";
    document.getElementById("shipping_zip_code").value = "14519";
    document.getElementById("shipping_instructions").value = "Don't drop it please!";

    document.getElementById("continue_payment").disabled = false;
    document.getElementById("continue_shipping").disabled = false;
};

ShoppingCart.prototype.emptyElement = function(element) {
    while(element.firstChild) {
        element.removeChild(element.firstChild);
    }
};
ShoppingCart.prototype.formatAddress = function(addr1,addr2,city,state,zip) {
    var div = document.createElement("div");
    var addr1Element = document.createElement("div");
    var addr2Element = document.createElement("div");
    var cityStateElement = document.createElement("div");

    div.className = "formatted_address";
    addr1Element.className = "address_addr_line_one";
    addr2Element.className = "address_addr_line_two";
    cityStateElement.className = "address_city_state";

    addr1Element.appendChild(document.createTextNode(addr1));
    addr2Element.appendChild(document.createTextNode(addr2));
    cityStateElement.appendChild(document.createTextNode(city+", "+state+" "+zip));

    div.appendChild(addr1Element);
    div.appendChild(addr2Element);
    div.appendChild(cityStateElement);

    return div;
};
ShoppingCart.prototype.parseTrackOne = function(trackData) {
    var regex = /^%B(\d{1,19})\^(.{2,26})\^(\d{2})(\d{2})(.{3})(.*)\?$/;
    var match = trackData.match(regex);
    if(match) {
        var data = new CreditCardTrackOneData();
        data.accountNumber = match[1];
        data.expirationMonth = (match[4]*1)-1;
        data.expirationYear = (match[3]*1)+2000;
        data.serviceCode = match[5];
        data.discretionaryData = match[6];

        var name = match[2];
        var idx;
        idx = name.indexOf("/");
        if(idx>=0) {
            data.lastName = name.substring(0,idx).trim();
            data.firstName = name.substring(idx+1).trim();
        } else {
            data.firstName = name;
            data.lastName = "";
        }
        return data;
    } else {
        return false;
    }
};
ShoppingCart.prototype.parseTrackTwo = function(trackData) {
    var regex = /^;(\d{1,19})=(\d{2})(\d{2})(\d{3})(.*)\?/;
    var match = trackData.match(regex);
    if(match) {
        var data = new CreditCardTrackTwoData();
        data.accountNumber = match[1];
        data.expirationMonth = (match[3]*1)-1;
        data.expirationYear = (match[2]*1)+2000;
        data.serviceCode = match[4];
        data.discretionaryData = match[5];
        return data;
    } else {
        return false;
    }
};
ShoppingCart.prototype.handleSocketData = function(con,data) {
    if(data._class==="ConnectionOpen") {
        if(data.serverInfo.devices.length>0) {
            var device = data.serverInfo.devices[0];
            con.send(JSON.stringify({"_class":"DeviceOpenRequest","uuid":device.uuid}));
        }
    } else if(data._class==="DeviceOpenResponse") {
        document.getElementById("device_connected").style.backgroundColor = "green";
        this.deviceConnected = true;

        this.canvas.width = data.device.displayInfo.width;
        this.canvas.height = data.device.displayInfo.height;
        this.ctx.fillStyle="white";
        this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);
        this.initializeDevice(data.device);
    } else if(data._class==="DeviceCloseResponse") {
        document.getElementById("device_connected").style.backgroundColor = "red";
        this.deviceConnected = false;
    } else if(data._class==="ConnectionDeviceMembershipChange") {
        if(!this.deviceConnected && data.devices.devices.length>0) {
            //See if we can connect to another device instead.
            var device = data.devices.devices[0];
            con.send(JSON.stringify({"_class":"DeviceOpenRequest","uuid":device.uuid}));   
        }
    } else if(data._class==="CardSwipe") {
        this.handleCardSwipe(data);
    } else if(data._class==="PenDown" || data._class==="PenMove" || data._class==="PenUp") {
        if(this.inReview) {
            //Look to see if we're pressing the button.
            var pressed = data.x>=30 && data.x<=210 && data.y>=52;
            var depressed = (data._class!=="PenUp") && pressed;
            if(this.buttonDepressed != depressed) {
                this.drawReviewScreen(pressed);
                this.pushScreen();
                this.buttonDepressed = depressed;
            }

            if(pressed && data._class==="PenUp") {
                this.inReview = false;
                this.inSign = true;
                this.drawSigningScreen();
                this.pushScreen();
                this.signCtx.fillStyle = "white";
                this.signCtx.fillRect(0,0,480,128);
                document.getElementById("agree_to_terms").checked = false;
                document.getElementById("agree_to_terms").onchange();
            }
        } else if(this.inSign) {
            var cancel = data.x>=0 && data.x<=40 && data.y>=16 && data.y<=24;
            var clear = data.x>=158 && data.x<=198 && data.y>=16 && data.y<=24;
            var accept = data.x>=200 && data.x<=240 && data.y>=16 && data.y<=24;

            if(data._class==="PenUp") {
                if(clear) {
                    this.pushScreen();
                    this.signCtx.fillStyle = "white";
                    this.signCtx.fillRect(0,0,480,128);
                } else if(cancel) {
                    this.inSign = false;
                    this.inReview = true;
                    this.drawReviewScreen();
                    this.pushScreen();
                    this.signCtx.fillStyle = "white";
                    this.signCtx.fillRect(0,0,480,128);
                    document.getElementById("agree_to_terms").checked = false;
                    document.getElementById("agree_to_terms").onchange();
                } else if(accept) {
                    this.inSign = false;
                    this.inReview = true;
                    this.drawReviewScreen();
                    this.pushScreen();
                    document.getElementById("agree_to_terms").checked = true;
                    document.getElementById("agree_to_terms").onchange();
                }
            }

            if(data._class==="PenDown") {
                this.lastPoint = data;
            }
            this.signCtx.beginPath();
            this.signCtx.moveTo(this.lastPoint.x*2,this.lastPoint.y*2);
            this.signCtx.lineTo(data.x*2,data.y*2);
            this.signCtx.stroke();

            this.lastPoint = data;
        }
    } else if(data._class==="RegionUpdateResponse" || data._class==="PushBitmapResponse") {
    } else {
        console.log(data);
    }
};
ShoppingCart.prototype.handleCardSwipe = function(data) {
    if(document.getElementById("cart_payment").style.display==="block") {
        //We're currently on the payment screen.
        var error = document.getElementById("card_validation_error");
        var cardNumber = document.getElementById("billing_card_number");
        var nameOnCard = document.getElementById("billing_name_on_card");
        var expirationMonth = document.getElementById("billing_expiration_month");
        var expirationYear = document.getElementById("billing_expiration_year");

        cardNumber.value = "";
        nameOnCard.value = "";
        expirationMonth.value = "";
        expirationYear.value = "";

        if(data.trackOneError) {
            this.emptyElement(error);
            error.appendChild(document.createTextNode("Problem reading card: "+data.trackOneError));
        } else if(data.format!=="financial") {
            this.emptyElement(error);
            error.appendChild(document.createTextNode("Card is not a financial card."));
        } else {
            var card = this.parseTrackOne(data.trackOne.trim());
            if(card) {
                cardNumber.value = card.accountNumber;
                nameOnCard.value = (card.firstName + " " + card.lastName).trim();
                expirationMonth.value = card.expirationMonth;
                expirationYear.value = card.expirationYear;

                document.getElementById("billing_card_number").onchange();
                document.getElementById("billing_information").onchange();
            } else {
                this.emptyElement(error);
                error.appendChild(document.createTextNode("Problem parsing track data."));
            }
        }
    }
};
ShoppingCart.prototype.drawScreen = function() {
    if(document.getElementById("cart_contents").style.display==="block") {
        this.drawCartScreen();
    } else if(document.getElementById("cart_payment").style.display==="block") {
        this.drawBillingScreen();
    } else if(document.getElementById("cart_shipping").style.display==="block") {
        this.drawShippingScreen();
    } else if(document.getElementById("cart_review").style.display==="block") {
        this.drawReviewScreen();
    } else if(document.getElementById("cart_thanks").style.display==="block") {
        this.drawThanksScreen();
    }
    this.pushScreen();
};
ShoppingCart.prototype.drawCartScreen = function() {
    this.ctx.fillStyle="white";
    this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);
    this.ctx.font = "7pt Silkscreen";
    this.ctx.fillStyle = "black";

    BlockFont.fillText(this.ctx,"Items in Cart",120,1,"center");
    this.ctx.lineWidth = 1;
    this.ctx.strokeRect(0.5,7.5,239,56);
    this.ctx.fillRect(0,7.5,239,8.5);
    this.ctx.fillRect(0,55,239,8.5);

    BlockFont.fillText(this.ctx,"Item:",2,9,"left",[0xFF,0xFF,0xFF]);
    BlockFont.fillText(this.ctx,"Qty:",165,9,"left",[0xFF,0xFF,0xFF]);
    BlockFont.fillText(this.ctx,"Price:",200,9,"left",[0xFF,0xFF,0xFF]);
    BlockFont.fillText(this.ctx,"Subtotal:",145,57,"left",[0xFF,0xFF,0xFF]);
    BlockFont.fillText(this.ctx,"$554.00",200,57,"left",[0xFF,0xFF,0xFF]);

    BlockFont.fillText(this.ctx,"Bosch 1812PSD 6-Inch Angle...",2,18);
    BlockFont.fillText(this.ctx,"1",165,18);
    BlockFont.fillText(this.ctx,"$158.00",200,18);

    BlockFont.fillText(this.ctx,"Bosch CS10 7-1/4-Inch Circ...",2,27);
    BlockFont.fillText(this.ctx,"1",165,27);
    BlockFont.fillText(this.ctx,"$88.00",200,27);

    BlockFont.fillText(this.ctx,"Bosch 1500C 16 Gauge Shear",2,36);
    BlockFont.fillText(this.ctx,"1",165,36);
    BlockFont.fillText(this.ctx,"$308.00",200,36);
};
ShoppingCart.prototype.drawBillingScreen = function() {
    this.ctx.fillStyle="white";
    this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);
    this.ctx.fillStyle = "black";

    BlockFont.fillText(this.ctx,"Billing Information",120,0,"center");
    
    BlockFont.fillText(this.ctx,"Card:",35,7,"right");
    BlockFont.fillText(this.ctx,"Type:",35,14,"right");
    BlockFont.fillText(this.ctx,"CVV2:",35,21,"right");
    BlockFont.fillText(this.ctx,"Name:",35,28,"right");
    BlockFont.fillText(this.ctx,"Expry:",35,35,"right");
    
    var monthSelect = document.getElementById("billing_expiration_month");
    var month = monthSelect.options[monthSelect.selectedIndex].text;
    var year = document.getElementById("billing_expiration_year").value;

    var type = this.checkCardType(document.getElementById("billing_card_number").value);
    type = (!type)?"Unknown":type;

    BlockFont.fillText(this.ctx,document.getElementById("billing_card_number").value,37,7,"left",[0,0,0],90);
    BlockFont.fillText(this.ctx,type,37,14,"left");
    BlockFont.fillText(this.ctx,document.getElementById("billing_card_verify").value,37,21,"left",[0,0,0],80);
    BlockFont.fillText(this.ctx,document.getElementById("billing_name_on_card").value,37,28,"left",[0,0,0],85);
    BlockFont.fillText(this.ctx,month+" "+year,37,35,"left");

    BlockFont.fillText(this.ctx,"Addr:",150,7,"right");
    BlockFont.fillText(this.ctx,"City:",150,21,"right");
    BlockFont.fillText(this.ctx,"State:",150,28,"right");
    BlockFont.fillText(this.ctx,"Zip:",150,35,"right");

    BlockFont.fillText(this.ctx,document.getElementById("billing_address_1").value,152,7,"left",[0,0,0],88);
    BlockFont.fillText(this.ctx,document.getElementById("billing_address_2").value,152,14,"left",[0,0,0],88);
    BlockFont.fillText(this.ctx,document.getElementById("billing_city").value,152,21,"left",[0,0,0],88);
    BlockFont.fillText(this.ctx,document.getElementById("billing_state").value,152,28,"left",[0,0,0],88);
    BlockFont.fillText(this.ctx,document.getElementById("billing_zip_code").value,152,35,"left",[0,0,0],88);


    this.ctx.fillRect(0,55,240,9);
    BlockFont.fillText(this.ctx,"Swipe card to fill in information.",120,57,"center",[0xFF,0xFF,0xFF]);
};
ShoppingCart.prototype.drawShippingScreen = function() {
    this.ctx.fillStyle="white";
    this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);
    this.ctx.fillStyle = "black";

    BlockFont.fillText(this.ctx,"Shipping Information",120,0,"center");

    BlockFont.fillText(this.ctx,"Addr:",37,7,"right");
    BlockFont.fillText(this.ctx,"City:",37,21,"right");
    BlockFont.fillText(this.ctx,"State:",37,28,"right");
    BlockFont.fillText(this.ctx,"Zip:",37,35,"right");

    BlockFont.fillText(this.ctx,document.getElementById("shipping_address_1").value,39,7,"left",[0,0,0],88);
    BlockFont.fillText(this.ctx,document.getElementById("shipping_address_2").value,39,14,"left",[0,0,0],88);
    BlockFont.fillText(this.ctx,document.getElementById("shipping_city").value,39,21,"left",[0,0,0],88);
    BlockFont.fillText(this.ctx,document.getElementById("shipping_state").value,39,28,"left",[0,0,0],88);
    BlockFont.fillText(this.ctx,document.getElementById("shipping_zip_code").value,39,35,"left",[0,0,0],88);

    BlockFont.fillText(this.ctx,"Delivery Instructions:",130,7);
    BlockFont.fillText(this.ctx,document.getElementById("shipping_instructions").value,130,14,"left",[0,0,0],110);
};
ShoppingCart.prototype.drawReviewScreen = function(invert) {
    this.ctx.fillStyle="white";
    this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);
    this.ctx.fillStyle = "black";

    var billingAddress1 = document.getElementById("billing_address_1").value;
    var billingAddress2 = document.getElementById("billing_address_2").value;
    var billingCity = document.getElementById("billing_city").value;
    var billingState = document.getElementById("billing_state").value;
    var billingZip = document.getElementById("billing_zip_code").value;

    var shippingAddress1 = document.getElementById("shipping_address_1").value;
    var shippingAddress2 = document.getElementById("shipping_address_2").value;
    var shippingCity = document.getElementById("shipping_city").value;
    var shippingState = document.getElementById("shipping_state").value;
    var shippingZip = document.getElementById("shipping_zip_code").value;
    var shippingInstructions = document.getElementById("shipping_instructions").value;

    var cardNumber = document.getElementById("billing_card_number").value;
    var nameOnCard = document.getElementById("billing_name_on_card").value;
    var cardExpirationMonth = document.getElementById("billing_expiration_month").value;
    var cardExpirationYear = document.getElementById("billing_expiration_year").value;
    var cardStr = this.checkCardType(cardNumber)+" ending in "+cardNumber.substring(cardNumber.length-4);

    var billingAddr = billingAddress1;
    billingAddr += (billingAddress2)?"\n"+billingAddress2+"\n":"\n";
    billingAddr += billingCity + ", "+billingState+" "+billingZip;

    var shippingAddr = shippingAddress1;
    shippingAddr += (shippingAddress2)?"\n"+shippingAddress2+"\n":"\n";
    shippingAddr += shippingCity + ", "+shippingState+" "+shippingZip;

    BlockFont.fillText(this.ctx,"Review Information",120,0,"center");
    BlockFont.fillText(this.ctx,"Bill to:",0,7);
    BlockFont.fillText(this.ctx,document.getElementById("billing_name_on_card").value,0,14,"left",[0,0,0],115);
    BlockFont.fillText(this.ctx,cardStr,0,21,"left",[0,0,0],115);
    BlockFont.fillText(this.ctx,billingAddr,0,28,"left",[0,0,0],115);

    BlockFont.fillText(this.ctx,"Ship to:",130,7);
    BlockFont.fillText(this.ctx,shippingAddr,130,14,"left",[0,0,0],110);

    BlockFont.fillText(this.ctx,"Total Amount: $567.00",130,42);

    this.ctx.strokeStyle = (invert)?"white":"black";
    this.ctx.fillStyle = (invert)?"black":"white";
    this.ctx.fillRect(30.5,51.5,180,12);
    this.ctx.strokeRect(30.5,51.5,180,12);
    BlockFont.fillText(this.ctx,"Accept and Sign",120,55,"center",((invert)?[0xFF,0xFF,0xFF]:[0,0,0]));
};
ShoppingCart.prototype.drawThanksScreen = function() {
    this.ctx.fillStyle="white";
    this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);
    this.ctx.fillStyle = "black";

    BlockFont.fillText(this.ctx,"Thanks!",120,0,"center");

    BlockFont.fillText(this.ctx,"Your payment is being processed. You will receive\nan e-mail notification when your order\nhas shipped with your tracking information.",120,25,"center",[0,0,0],240);
};
ShoppingCart.prototype.drawSigningScreen = function() {
    this.ctx.fillStyle="white";
    this.ctx.fillRect(0,0,this.canvas.width,this.canvas.height);
    this.ctx.fillStyle = "black";

    this.ctx.strokeStyle = "black";

    BlockFont.fillText(this.ctx,"I understand that by signing I agree to pay the amount\nof $567.00 to the retailer.",0,0);
    this.ctx.strokeRect(0.5,16.5,239,47);

    BlockFont.fillText(this.ctx,"Cancel",20,18,"center");
    this.ctx.strokeRect(0.5,16.5,40,8);

    BlockFont.fillText(this.ctx,"Accept",220,18,"center");
    this.ctx.strokeRect(200.5,16.5,40,8);

    BlockFont.fillText(this.ctx,"Clear",178,18,"center");
    this.ctx.strokeRect(158.5,16.5,40,8);

    var obj = {
        "_class": "RegionUpdateRequest",
        "region": {
            "_class": "Region",
            "id": 3,
            "x1": 0,
            "y1": 16,
            "x2": 239,
            "y2": 63,
            "containerType": {
                "flags": {
                    "inkEnabled": true
                }
            }
        }
    };
    this.socket.send(JSON.stringify(obj));

    this.ctx.beginPath();
    this.ctx.moveTo(5,50);
    this.ctx.lineTo(10,55);
    this.ctx.moveTo(5,55);
    this.ctx.lineTo(10,50);
    this.ctx.moveTo(15,54.5);
    this.ctx.lineTo(240,54.5);
    this.ctx.stroke();
};
ShoppingCart.prototype.pushScreen = function() {
    var obj = {
        "_class":"PushBitmapRequest",
        "x":0,
        "y":0,
        "data":this.canvas.toDataURL("image/png"),
        "bitDepth":1
    };
    this.socket.send(JSON.stringify(obj));
};
ShoppingCart.prototype.initializeDevice = function(device) {
    var x;
    //Blank regions.
    this.socket.send(JSON.stringify({
        "_class":"RegionUpdateRequest",
        "region": {
            "id": 2,
            "x1": 0,
            "x2": device.displayInfo.width-1,
            "y1": 0,
            "y2": device.displayInfo.height-1,
            "containerType": {
                "_class":"RegionContainer",
                "flags": {
                    "inkEnabled": false
                }
            },
            "flags": {
                "acceptsTouch": true,
                "decodeRegion": true,
                "hasFrame": false,
                "inUse": true,
                "visible": true
            }
        }
    }));
    for(x=2;x<device.regions.length;x++) {
        this.socket.send(JSON.stringify({
            "_class":"RegionUpdateRequest",
            "region": {
                "id": (x+1),
                "x1": 0,
                "x2": 1,
                "y1": 0,
                "y2": 1,
                "containerType": {"_class":"RegionContainer"},
                "flags": {
                    "acceptsTouch": false,
                    "decodeRegion": false,
                    "hasFrame": false,
                    "inUse": false,
                    "visible": false
                }
            }
        }));
    };
    this.drawScreen();
};
function CreditCardTrackOneData() {
    this.accountNumber = "";
    this.firstName = "";
    this.lastName = "";
    this.expirationMonth = -1;
    this.expirationYear = -1;
    this.serviceCode = "";
    this.discretionaryData = "";
}
function CreditCardTrackTwoData() {
    this.accountNumber = "";
    this.expirationMonth = -1;
    this.expirationYear = -1;
    this.serviceCode = "";
    this.discretionaryData = "";
}