function ScriptelApi(url) {
    "use strict";
    if (!url) {
        var ssl = document.location.protocol.indexOf("https") >= 0;
        url = (ssl ? "wss://" : "ws://") + document.location.host + "/socket";
    }
    this.url = url;

    this.socket = false;
    this.socketOpen = false;
    this.closedOnPurpose = false;

    this.device = false;
    this.deviceOpen = false;

    this.callbacks = [];

}
ScriptelApi.prototype.open = function () {
    "use strict";
    this.socket = new WebSocket(this.url);
    var t = this;
    this.socket.onopen = function () {
        t.onConnectionOpen();
    };
    this.socket.onclose = function (evt) {
        t.onConnectionClose(evt);
    };
    this.socket.onerror = function (evt) {
        t.onSocketError(evt);
    };
    this.socket.onmessage = function (msg) {
        t.readSocketData(msg);
    };
    this.closedOnPurpose = false;
};
ScriptelApi.prototype.close = function () {
    "use strict";
    this.socket.close();
    this.socketOpen = false;
    this.closedOnPurpose = true;
};
ScriptelApi.prototype.reconnect = function () {
    "use strict";
};
ScriptelApi.prototype.readSocketData = function (msg) {
    "use strict";
    var data = JSON.parse(msg.data);
    this.notify(data._class, data);
};
ScriptelApi.prototype.openDevice = function (uuid) {
    "use strict";
    var obj = {};
    obj.uuid = uuid;
    obj._class = "DeviceOpenRequest";
    this.deviceOpen = true;
    this.socket.send(JSON.stringify(obj));
};
ScriptelApi.prototype.closeDevice = function () {
    "use strict";
    var obj = {};
    obj._class = "DeviceCloseRequest";
    this.deviceOpen = false;
    this.socket.send(JSON.stringify(obj));
};
ScriptelApi.prototype.onConnectionClose = function (evt) {
    "use strict";
    this.notify("ConnectionClosed", evt);
    this.socketOpen = false;
    this.deviceOpen = false;
    this.device = false;

    //Attempt to gracefully reconnect
    if (!this.reconnecting) {
        if (!this.socketOpen && !this.closedOnPurpose) {
			this.reconnecting = true;
			this.open();
		}
    }
};
ScriptelApi.prototype.onConnectionOpen = function () {
    "use strict";
    this.socketOpen = true;
    this.reconnecting = false;
};
ScriptelApi.prototype.onSocketError = function (evt) {
    "use strict";
    this.notify("ConnectionError", evt);

	if (this.reconnecting) {
		this.open();
	}
};
ScriptelApi.prototype.registerCallback = function (func) {
    "use strict";
    this.callbacks[this.callbacks.length] = func;
};
ScriptelApi.prototype.notify = function (eventType, data) {
    "use strict";
    var x;
    for (x = 0; x < this.callbacks.length; x++) {
        this.callbacks[x](eventType, data);
    }
};
ScriptelApi.prototype.scanForServers = function () {
    "use strict";
    if (!this.socketOpen) {
        throw "Web socket to server isn't currently open.";
    }

	var obj = {"_class": "Enumerate"};
	this.socket.send(JSON.stringify(obj));
};
ScriptelApi.prototype.uploadFirmware = function (firmware) {
    "use strict";
    this.socket.send(JSON.stringify(firmware));
};
ScriptelApi.prototype.forceRecalibration = function () {
    "use strict";
    this.socket.send(JSON.stringify({"_class": "ForceRecalibrate"}));
};
ScriptelApi.prototype.updateRegion = function (region) {
    "use strict";

    if(region.buttonType || region.containerType) {
        //Need to test for correct bounding.
        var tmp;
        if(region.x1 > region.x2) {
            tmp = region.x1;
            region.x1 = region.x2;
            region.x2 = tmp;
        }

        if(region.y1 > region.y2) {
            tmp = region.y1;
            region.y1 = region.y2;
            region.y2 = tmp;
        }
    }

    this.socket.send(JSON.stringify({"_class": "RegionUpdateRequest", "region": region}));
};
ScriptelApi.prototype.updateRenderSettings = function (settings) {
    "use strict";
    this.socket.send(JSON.stringify({"_class": "RenderSettingsUpdateRequest", "renderSettings": settings}));
};

if (!console) {
    console = {};
}
if (!console.log) {
    "use strict";
    console.log = function () {};
}
