function WaitingIndicator(fillColor, orbCount) {
    "use strict";
    this.fillColor = fillColor;
    this.orbCount = orbCount;
    this.orbs = false;
    this.orbStates = false;
    this.element = false;
    this.container = false;
    this.time = 1000 / orbCount;
}
WaitingIndicator.prototype.attach = function (element) {
    "use strict";
    this.element = element;
    this.orbs = [];
    this.orbStates = [];

    var size = (element.offsetWidth > element.offsetHeight) ? element.offsetHeight : element.offsetWidth;
    var maxOrbSize = size / (this.orbCount / 2);
    var minOrbSize = (size / 1.5) / this.orbCount;
    this.container = document.createElement("div");
    this.container.style.position = "relative";
    this.container.style.width = "100%";
    this.container.style.height = "100%";
    this.container.style.top = "50%";
    this.container.style.left = "50%";
    var r = (size / 2) - (maxOrbSize / 2) - 2;

    var t = this;
    var x;

    var transEndFunc = function (evt) {
        if (evt.propertyName === "width") {
            var idx = t.orbs.indexOf(this);
            var orb = this;
            var angle = ((2 * Math.PI) / t.orbCount) * idx;
            if (t.orbStates[idx]) {
                orb.style.width = parseInt(minOrbSize, 10) + "px";
                orb.style.height = parseInt(minOrbSize, 10) + "px";
                orb.style.left = ((Math.cos(angle) * r) - (minOrbSize / 2)) + "px";
                orb.style.top = ((Math.sin(angle) * r) - (minOrbSize / 2)) + "px";
            } else {
                orb.style.width = parseInt(maxOrbSize, 10) + "px";
                orb.style.height = parseInt(maxOrbSize, 10) + "px";
                orb.style.left = ((Math.cos(angle) * r) - (maxOrbSize / 2)) + "px";
                orb.style.top = ((Math.sin(angle) * r) - (maxOrbSize / 2)) + "px";
            }
            t.orbStates[idx] = !t.orbStates[idx];
        }
    };

    for (x = 0; x < this.orbCount; x++) {
        var angle = ((2 * Math.PI) / this.orbCount) * x;
        var orb = document.createElement("div");
        orb.style.position = "absolute";
        orb.style.backgroundColor = this.fillColor;
        orb.style.borderRadius = maxOrbSize  + "px";
        orb.style.width = maxOrbSize + "px";
        orb.style.height = maxOrbSize + "px";
        orb.style.left = ((Math.cos(angle) * r) - (maxOrbSize / 2)) + "px";
        orb.style.top = ((Math.sin(angle) * r) - (maxOrbSize / 2)) + "px";

        //Animation
        orb.style.transitionProperty = "left,top,width,height";
        orb.style.transitionDuration = ".5s";

        orb.addEventListener("transitionend", transEndFunc, false);

        this.orbs[x] = orb;
        this.orbStates[x] = false;
        this.container.appendChild(orb);
    }
    this.element.appendChild(this.container);

    var o = 0;
    var start = function () {
        if (o < t.orbCount) {
            var orb = t.orbs[o];
            var angle = ((2 * Math.PI) / t.orbCount) * o;
            orb.style.width = parseInt(minOrbSize, 10) + "px";
            orb.style.height = parseInt(minOrbSize, 10) + "px";
            orb.style.left = ((Math.cos(angle) * r) - (minOrbSize / 2)) + "px";
            orb.style.top = ((Math.sin(angle) * r) - (minOrbSize / 2)) + "px";
            o++;
            setTimeout(start, t.time);
        }
    };
    setTimeout(start, 1);
};
WaitingIndicator.prototype.destroy = function () {
    "use strict";
    this.element.removeChild(this.container);
};
