function ScriptelWebapp() {
    "use strict";
    var t = this;
    this.api = new ScriptelApi();
    this.api.registerCallback(function (type, data) {
        t.eventHandler(type, data);
    });
    this.api.open();

    this.serverConnection = false;
    this.deviceConnection = false;
    this.deviceInfo = false;
    this.serverDevices = false;
    this.discoveryTimeout = 2000;
    this.discoveredServers = [];

    this.serverWaitingIndicator = false;
    this.emulatorScale = 2;
    this.ctx = false;
    this.penDown = false;
    this.lastPoint = false;

    this.firmwareUpdate = false;
    this.editorElements = false;
    this.registerEventHandlers();

    this.checkForBrowserSupport();
}
ScriptelWebapp.prototype.checkForBrowserSupport = function() {
    "use strict";
    var warning = document.getElementById("unsupported_warning");
    var warningCover = document.getElementById("modal_coverup");
    var override = document.getElementById("unsupported_override");

    override.disabled = false;
    var canvas = document.createElement("canvas");

    override.addEventListener("click", function() {
        warningCover.parentNode.removeChild(warningCover);
        warning.parentNode.removeChild(warning);
    }, false);

    if(window.WebSocket) {
        if(canvas.getContext && canvas.getContext("2d")) {
            if(window.addEventListener) {
                warningCover.parentNode.removeChild(warningCover);
                warning.parentNode.removeChild(warning);
            }
        }
    }
};
ScriptelWebapp.prototype.clearAll = function () {
    "use strict";
    //Clear Status Tab
    this.empty(document.getElementById("device_status_product_name"));
    this.empty(document.getElementById("device_status_uuid"));
    this.empty(document.getElementById("device_status_serial_number"));
    this.empty(document.getElementById("device_status_aspect_ratio"));
    this.empty(document.getElementById("device_status_screen_resolution"));
    this.empty(document.getElementById("device_status_color_depth"));
    this.empty(document.getElementById("device_status_supported_regions"));
    this.empty(document.getElementById("device_status_digitizer_size"));

    //Clear Firmware Tab
    document.getElementById("firmware_file").value = "";
    document.getElementById("firmware_upload_local").disabled = true;
    document.getElementById("firmware_update_progress").value = 0;
    document.getElementById("device_firmware_internet_version").options[document.getElementById("device_firmware_internet_version").options.length] = new Option("Loading...");
    document.getElementById("device_firmware_internet_version").options.length = 0;
    document.getElementById("device_firmware_internet_version").disabled = true;
    document.getElementById("firmware_download_internet_button").disabled = true;
    document.getElementById("device_firmware_source_internet_radio").checked = true;

    document.getElementById("device_firmware_source_internet").style.display = "block";
    document.getElementById("device_firmware_source_local").style.display = "none";

    this.empty(document.getElementById("device_firmware_current_version"));
    this.empty(document.getElementById("device_firmware_latest_version"));

    //Clear Calibration Tab
    this.empty(document.getElementById("device_currently_calibrating"));
    this.empty(document.getElementById("device_calibration_point"));

    //Clear Screen Emulator
    if (this.ctx) {
        this.ctx.fillColor = "white";
        this.ctx.fillRect(0, 0, this.ctx.canvas.width, this.ctx.canvas.height);
    }
};
ScriptelWebapp.prototype.registerEventHandlers = function () {
    "use strict";
    var t = this;
    var tabs = document.getElementsByClassName("device_tab");
    var x;
    var switchFunction = function () {
        t.switchTab(this);
    };

    for (x = 0; x < tabs.length; x++) {
        var tab = tabs[x];
        tab.addEventListener("click", switchFunction, false);
    }


    var uploadButton = document.getElementById("firmware_upload_local");
    uploadButton.addEventListener("click", function () {
        if (confirm("Are you sure you want to update the firmware on this device?\nPlease do not unplug the device while firmware is updating.")) {
            t.api.uploadFirmware(t.firmwareUpdate);
        }
    }, false);
    
    var displayFile = document.getElementById("display_file");
    var displayFileImage = document.getElementById("display_file_image");
    var displayUploadButton = document.getElementById("display_upload_button");
    displayFile.addEventListener("change", function (evt) {
        if (evt.target.files.length > 0) {
            var file = evt.target.files[0];
            var reader = new FileReader();
            reader.onload = function (evt) {
                displayFileImage.src = evt.target.result;
                displayUploadButton.disabled = false;
            };
            reader.readAsDataURL(file);
        }
    }, false);
    displayUploadButton.addEventListener("click", function (evt) {
        var destWidth = displayFileImage.width;
        var destHeight = displayFileImage.height;
        var scaleToFit = document.getElementById("display_file_scale_to_fit").checked;

        if(scaleToFit) {
            var xScale = t.deviceInfo.displayInfo.width / displayFileImage.width;
            var yScale = t.deviceInfo.displayInfo.height / displayFileImage.height;
            var scale = (xScale < yScale) ? xScale : yScale;

            destWidth *= scale;
            destHeight *= scale;

        }


        var canvas = document.createElement("canvas");
        var ctx = canvas.getContext("2d");
        canvas.width = destWidth;
        canvas.height = destHeight;
        ctx.drawImage(displayFileImage,0,0,destWidth,destHeight);
        var url = canvas.toDataURL("image/png");
        
        var xPos = document.getElementById("display_file_x");
        var yPos = document.getElementById("display_file_y");
        var depth = document.getElementById("display_file_depth");
        
        var obj = {"_class":"PushBitmapRequest","x":xPos.value,"y":yPos.value,"bitDepth":depth.value*1,"data":url};
        t.api.socket.send(JSON.stringify(obj));
    }, false);

    //Now for the firmware file handle
    var firmwareFile = document.getElementById("firmware_file");
    firmwareFile.addEventListener("change", function (evt) {
        if (evt.target.files.length > 0) {
            var file = evt.target.files[0];
            t.firmwareUpdate = {"_class": "FirmwareUpdateRequest", "update": []};
            var reader = new FileReader();
            reader.onload = function (evt) {
                t.firmwareUpdate.update = evt.target.result;
                uploadButton.disabled = false;
            };
            reader.readAsDataURL(file);
        }
    }, false);

    //Calibration
    var recalButton = document.getElementById("device_recalibrate");
    recalButton.addEventListener("click", function () {
        t.api.forceRecalibration();
    }, false);

    var updateRender = document.getElementById("device_render_update");
    updateRender.addEventListener("click", function () {
        var s = {
            "_class": "RenderSettings",
            "type": document.getElementById("device_render_type").value,
            "scale": document.getElementById("device_render_scale").value,
            "penStyle": {
                "_class": "PenStyle",
                "options": {
                    "strokeStyle": document.getElementById("device_render_stroke_color").value
                },
                "renderFunction": document.getElementById("device_pen_type").value
            },
            "crop": document.getElementById("device_render_autocrop").checked
        };
        t.api.updateRenderSettings(s);
    }, false);

    var firmwareInternetRadio = document.getElementById("device_firmware_source_internet_radio");
    var firmwareLocalRadio = document.getElementById("device_firmware_source_local_radio");

    var firmwareSwitch = function() {
        var firmwareInternet = document.getElementById("device_firmware_source_internet");
        var firmwareLocal = document.getElementById("device_firmware_source_local");

        if(firmwareInternetRadio.checked) {
            firmwareInternet.style.display = "block";
            firmwareLocal.style.display = "none";
        } else {
            firmwareInternet.style.display = "none";
            firmwareLocal.style.display = "block";
        }
    };
    firmwareInternetRadio.addEventListener("change", firmwareSwitch, false);
    firmwareLocalRadio.addEventListener("change", firmwareSwitch, false);
    

    document.getElementById("firmware_download_internet_button").addEventListener("click", function() {
        var select = document.getElementById("device_firmware_internet_version");
        var url = select.options[select.selectedIndex].value;
        t.getInternetFirmware(url);
    }, false);

    var brightnessSlider = document.getElementById("device_display_brightness");
    var contrastSlider = document.getElementById("device_display_contrast");

    brightnessSlider.addEventListener("change", function() {
        t.deviceInfo.displaySettings.brightness = brightnessSlider.value;
        t.updateDisplaySettings();
    }, false);

    contrastSlider.addEventListener("change", function() {
        t.deviceInfo.displaySettings.contrast = contrastSlider.value;
        t.updateDisplaySettings();
    }, false);

    document.getElementById("device_display_show_contrast").addEventListener("click", function() {
        var canvas = document.createElement("canvas");

        var devWidth = t.deviceInfo.displayInfo.width;
        var devHeight = t.deviceInfo.displayInfo.height;

        canvas.width = devWidth;
        canvas.height = devHeight;

        var ctx = canvas.getContext("2d");
        var grd = ctx.createLinearGradient(0, 0, devWidth, 0);
        grd.addColorStop(0, "white");
        grd.addColorStop(1, "black");

        ctx.fillStyle = grd;
        ctx.fillRect(0, 0, devWidth, devHeight);

        var obj = {"_class":"PushBitmapRequest","x":0, "y":0,"bitDepth":4,"data":canvas.toDataURL("image/png")};
        t.api.socket.send(JSON.stringify(obj));
    }, false);

    document.getElementById("device_region_editor_restore_default").addEventListener("click", function() {
        "use strict";
        t.api.socket.send(JSON.stringify({"_class":"ResetDefaultRegionsRequest"}));
    }, false);
};

ScriptelWebapp.prototype.updateDisplaySettings = function() {
    var req = {
        "_class":"UpdateDisplaySettingsRequest",
        "red": this.deviceInfo.displaySettings.red,
        "green": this.deviceInfo.displaySettings.green,
        "blue": this.deviceInfo.displaySettings.blue,
        "contrast": this.deviceInfo.displaySettings.contrast * 1,
        "brightness": this.deviceInfo.displaySettings.brightness * 1
    };
    this.api.socket.send(JSON.stringify(req));
}

ScriptelWebapp.prototype.switchTab = function (tab) {
    "use strict";
    var idx = tab.className.indexOf("device_tab_disabled");
    if(idx < 0) {
        var id = tab.id.substring(tab.id.lastIndexOf("_") + 1);
        var body = document.getElementById("device_body_" + id);
        var x;

        var selected = document.getElementsByClassName("device_tab_body_selected");
        for (x = 0; x < selected.length; x++) {
            selected[x].className = selected[x].className.replace("device_tab_body_selected", "");
        }
        body.className += " device_tab_body_selected";

        selected = document.getElementsByClassName("device_tab_selected");
        for (x = 0; x < selected.length; x++) {
            selected[x].className = selected[x].className.replace("device_tab_selected", "");
        }
        tab.className += " device_tab_selected";
    }
};

ScriptelWebapp.prototype.initializeScreenEmulator = function() {
    "use strict";
    var canvas = document.getElementById("device_screen_emulator_canvas");
    this.display = new ScriptelDisplay(canvas, this.deviceInfo.displayInfo.width, this.deviceInfo.displayInfo.height, this.emulatorScale);
    this.display.setRegions(this.deviceInfo.regions);

    this.clearEditorElements();
    this.disableEditorElements(true);

    var t = this;
    this.display.selectListeners[this.display.selectListeners.length] = function(id) {
        "use strict";
        t.selectTreeRegion(id);
    };

    this.display.regionUpdateListeners[this.display.regionUpdateListeners.length] = function(region, doneDragging) {
        "use strict";
        t.deviceInfo.regions[region.id - 1] = region;
        t.selectRegion(region.id);

        if(doneDragging) {
            t.api.updateRegion(region);
        }
    };

    this.display.activeContainerListeners[this.display.activeContainerListeners.length] = function(id) {
        "use strict";
        t.api.updateRegion(t.deviceInfo.regions[id - 1]);
        t.updateActiveContainerMarker();
    };
};

ScriptelWebapp.prototype.updateActiveContainerMarker = function() {
    "use strict";
    if(this.display.activeContainer <= 0) {
        return;
    }

    var items = document.getElementsByClassName("device_region_tree_item");
    var i;
    for(i = 0; i < items.length; i++) {
        items[i].className = items[i].className.replace(" device_region_tree_active_container", "");
    }
    var activeContainer = document.getElementById("device_region_tree_item_" + this.display.activeContainer);
    activeContainer.className += " device_region_tree_active_container";
};

ScriptelWebapp.prototype.eventHandler = function (type, data) {
    "use strict";
    if (type === "ConnectionOpen") {
        this.serverConnection = true;
        this.updateStatus();

        this.api.scanForServers();
        this.invalidateServerList();

        this.serverDevices = data.serverInfo;
        this.updateStatus();
    } else if (type === "ConnectionDeviceMembershipChange") {
        this.serverDevices = data.devices;
        this.updateStatus();
    } else if (type === "ConnectionClosed") {
        this.serverConnection = false;
        this.deviceConnection = false;
        this.updateStatus();
    } else if (type === "Instance") {
        this.registerDiscoveredServer(data);
        this.updateServerList();
    } else if (type === "DeviceOpenResponse") {
        this.deviceInfo = data.device;
        this.deviceConnection = true;
        this.processDeviceInfo(data.device);
        this.initializeScreenEmulator();
        this.updateStatus();
        this.updateServerList();
        this.drawRegionTree();
    } else if (type === "PenDown") {
        this.display.penDown(data.x, data.y);
    } else if (type === "PenMove") {
        this.display.penMove(data.x, data.y);
    } else if (type === "PenUp") {
        this.display.penUp(data.x, data.y);
    } else if (type === "ButtonDown") {
        this.display.invertButton = data.id;
        this.display.redraw();
    } else if (type === "ButtonUp") {
        this.display.invertButton = -1;
        this.display.redraw();
    } else if (type === "ButtonPress") {
        this.display.invertButton = -1;
        this.display.redraw();
    } else if (type === "DeviceCloseResponse") {
        this.deviceConnection = false;
        this.updateStatus();
    } else if (type === "FirmwareUpdateNotice") {
        var element = document.getElementById("firmware_update_progress");
        element.value = data.percent * 100;
    } else if (type === "CalibrationUpdate") {
        var currentlyCalibrating = document.getElementById("device_currently_calibrating");
        var calibrationPoint = document.getElementById("device_calibration_point");
        this.empty(currentlyCalibrating);
        this.empty(calibrationPoint);

        if (!data.done) {
            currentlyCalibrating.appendChild(document.createTextNode("True"));
            calibrationPoint.appendChild(document.createTextNode(data.point));
        } else {
            currentlyCalibrating.appendChild(document.createTextNode("False"));
            calibrationPoint.appendChild(document.createTextNode("Center"));
        }
        this.deviceInfo.calibrationMode.calibrating = !data.done;
        this.deviceInfo.calibrationMode.calibrationPoint = data.point;

        this.display.calibrating = this.deviceInfo.calibrationMode.calibrating;
        this.display.calibrationPoint = this.deviceInfo.calibrationMode.calibrationPoint;
        this.display.redraw();
    } else if (type === "ScriptelException") {
        alert(data.message);
    } else if (type === "DeviceContention" ) {
        var iconNew = 'faviconAlert.ico';
        document.getElementById('favicon').href = iconNew;
        alert("A device in use by the toolbox is being requested by another application.  This device cannot be used in the requesting application until it is closed in the toolbox.");
        var iconOld = 'favicon.ico';
        document.getElementById('favicon').href = iconOld;
    } else if (type === "RegionUpdateResponse") {
        //Region was updated, lets update it in our cache and redraw.
        var region = data.region;
        this.deviceInfo.regions[region.id - 1] = region;
        this.display.regions[region.id - 1] = region;
        if(region.id === this.display.selectedRegion) {
            this.display.setRegion(region);
            this.selectTreeRegion(region.id);
        }
    } else if (type === "RenderedImage") {
        var img = document.getElementById("device_rendered_image");
        var div = document.getElementById("device_last_render");
        div.style.display="block";
        img.width = data.width;
        img.height = data.height;
        img.src = data.data;
    } else if (type === "CardSwipe") {
        var trackOne = document.getElementById("device_cardreader_trackone");
        var trackTwo = document.getElementById("device_cardreader_tracktwo");
        var trackThree = document.getElementById("device_cardreader_trackthree");

        trackOne.value = data.trackOneError ? data.trackOneError : data.trackOne;
        trackTwo.value = data.trackTwoError ? data.trackTwoError : data.trackTwo;
        trackThree.value = data.trackThreeError ? data.trackThreeError : data.trackThree;
    } else if (type === "ResetDefaultRegionsResponse") {
        this.deviceInfo.regions = data.regions;
        this.display.setRegions(data.regions);
        this.selectTreeRegion(-1);
        this.drawRegionTree();
    } else {
        console.log(type, data);
    }
};

ScriptelWebapp.prototype.updateStatus = function () {
    "use strict";
    var serverElement = document.getElementById("server_connection_status");
    var deviceElement = document.getElementById("device_connection_status");
    var x;

    while (serverElement.firstChild) {
        serverElement.removeChild(serverElement.firstChild);
    }
    while (deviceElement.firstChild) {
        deviceElement.removeChild(deviceElement.firstChild);
    }

    if (this.serverConnection) {
        if (this.serverDevices) {
            serverElement.appendChild(document.createTextNode("Active (" + this.serverDevices.hostName + ")"));
        } else {
            serverElement.appendChild(document.createTextNode("Active"));
        }
        serverElement.className = "status_active";
    } else {
        serverElement.appendChild(document.createTextNode("Inactive"));
        serverElement.className = "status_inactive";
    }

    if (this.deviceConnection) {
        var dev = false;
        for (x = 0; x < this.serverDevices.devices.length && !dev; x++) {
            if (this.serverDevices.devices[x].uuid === this.deviceInfo.uuid) {
                dev = this.serverDevices.devices[x];
            }
        }

        if (dev) {
            deviceElement.appendChild(document.createTextNode("Active (" + dev.product + ")"));
            deviceElement.setAttribute("title", "Device UUID: " + this.deviceInfo.uuid);
        } else {
            deviceElement.appendChild(document.createTextNode("Active"));
        }
        deviceElement.className = "status_active";
        document.getElementById("device_editor_blocker").style.display = "none";
    } else {
        deviceElement.appendChild(document.createTextNode("Inactive"));
        deviceElement.className = "status_inactive";
        document.getElementById("device_editor_blocker").style.display = "block";
        this.clearAll();
    }
};
ScriptelWebapp.prototype.invalidateServerList = function () {
    "use strict";
    var e = document.getElementById("server_browser");
    while (e.firstChild !== e.lastChild) {
        e.removeChild(e.lastChild);
    }

    var indicator = document.createElement("div");
    indicator.className = "server_browser_status_indicator";
    e.appendChild(indicator);

    this.serverWaitingIndicator = new WaitingIndicator("#808080", 8);
    this.serverWaitingIndicator.attach(indicator);

    var status = document.createElement("div");
    status.appendChild(document.createTextNode("Scanning..."));
    status.className = "server_browser_status_message";
    e.appendChild(status);
};
ScriptelWebapp.prototype.updateServerList = function () {
    "use strict";
    var x;
    var y;
    var t = this;
    var e = document.getElementById("server_browser");
    var serverClick = function () {
        var newServer = t.discoveredServers[this.getAttribute("idx")];
        t.api.close();
        var url = false;
        if (newServer.socketPort) {
            url = "ws://" + newServer.address + ":" + newServer.socketPort + "/";
        } else {
            url = "wss://" + newServer.address + ":" + newServer.sslSocketPort + "/";
        }
        t.api = new ScriptelApi(url);
        t.api.registerCallback(function (type, data) {
            t.eventHandler(type, data);
        });
        t.api.open();
    };
    var deviceClick = function () {
        t.openDevice(this.getAttribute("uuid"));
    };

    if (this.serverWaitingIndicator) {
        this.serverWaitingIndicator.destroy();
        this.serverWaitingIndicator = false;
    }
    while (e.firstChild !== e.lastChild) {
        e.removeChild(e.lastChild);
    }
    for (x = 0; x < this.discoveredServers.length; x++) {
        var server = this.discoveredServers[x];
        var s = document.createElement("div");
        s.className = "server_browser_server";


        var label = document.createElement("div");
        label.className = "server_browser_server_label";
        label.appendChild(document.createTextNode(server.hostName));
        s.appendChild(label);
        label.setAttribute("title", "Server Address: " + server.address);
        label.setAttribute("idx", x);

        var count = document.createElement("div");
        count.className = "server_browser_server_devcount";
        count.appendChild(document.createTextNode(server.devices.length + " devices"));
        label.appendChild(count);
        e.appendChild(s);

        label.addEventListener("click", serverClick, false);

        if (server.hostName === this.serverDevices.hostName) {
            label.className += " server_browser_server_connected";
            var devList = document.createElement("ul");
            devList.className = "server_browser_devlist";

            for (y = 0; y < this.serverDevices.devices.length; y++) {
                var dev = this.serverDevices.devices[y];
                var li = document.createElement("li");
                li.setAttribute("title", "Device UUID: " + dev.uuid);
                li.setAttribute("uuid", dev.uuid);
                li.appendChild(document.createTextNode(dev.product));
                devList.appendChild(li);

                li.addEventListener("click", deviceClick, false);

                if (this.deviceConnection && dev.uuid === this.deviceInfo.uuid) {
                    li.className += " server_browser_device_connected";
                }
            }
            s.appendChild(devList);
        }
    }
};
ScriptelWebapp.prototype.openDevice = function (uuid) {
    "use strict";
    if (this.deviceConnection) {
        this.api.closeDevice();
    }
    this.api.openDevice(uuid);
};
ScriptelWebapp.prototype.registerDiscoveredServer = function (data) {
    "use strict";
    var found = false;
    var x;
    for (x = 0; x < this.discoveredServers.length && !found; x++) {
        if (data.address === this.discoveredServers[x].address || data.hostName === this.discoveredServers[x].hostName) {
            found = true;
            //This is an update for an already discovered server.
            this.discoveredServers[x] = data;
        }
    }
    if (!found) {
        this.discoveredServers[this.discoveredServers.length] = data;
    }
};
ScriptelWebapp.prototype.processDeviceInfo = function (data) {
    "use strict";
    var x;
    var productName = document.getElementById("device_status_product_name");
    var uuid = document.getElementById("device_status_uuid");
    var firmwareVersion = document.getElementById("device_status_firmware_version");
    var firmwareVersionCurrent = document.getElementById("device_firmware_current_version");
    var serialNumber = document.getElementById("device_status_serial_number");
    var aspectRatio = document.getElementById("device_status_aspect_ratio");
    var screenResolution = document.getElementById("device_status_screen_resolution");
    var colorDepthStatus = document.getElementById("device_status_color_depth");
    var supportedRegions = document.getElementById("device_status_supported_regions");
    var digitizerSize = document.getElementById("device_status_digitizer_size");

    var currentlyCalibrating = document.getElementById("device_currently_calibrating");
    var calibrationPoint = document.getElementById("device_calibration_point");

    this.empty(productName);
    this.empty(uuid);
    this.empty(firmwareVersion);
    this.empty(serialNumber);
    this.empty(aspectRatio);
    this.empty(screenResolution);
    this.empty(colorDepthStatus);
    this.empty(supportedRegions);
    this.empty(digitizerSize);

    this.empty(currentlyCalibrating);
    this.empty(calibrationPoint);

    var devInfo = false;
    for (x = 0; x < this.serverDevices.devices.length && !devInfo; x++) {
        if (this.serverDevices.devices[x].uuid === data.uuid) {
            devInfo = this.serverDevices.devices[x];
        }
    }
    productName.appendChild(document.createTextNode(devInfo.manufacturer + " " + devInfo.product + " (" + devInfo.vendorId + ":" + devInfo.productId + ")"));
    uuid.appendChild(document.createTextNode(data.uuid));
    firmwareVersion.appendChild(document.createTextNode(data.softwareVersion.firmwareVersion.major + "." + data.softwareVersion.firmwareVersion.minor + "." + data.softwareVersion.firmwareVersion.release));
    firmwareVersionCurrent.appendChild(document.createTextNode(data.softwareVersion.firmwareVersion.major + "." + data.softwareVersion.firmwareVersion.minor + "." + data.softwareVersion.firmwareVersion.release));
    serialNumber.appendChild(document.createTextNode(data.serialNumber ? data.serialNumber : "Unset"));
    
    screenResolution.appendChild(document.createTextNode(data.displayInfo.width + "x" + data.displayInfo.height));

    var depth = "N/A";
    if(data.capabilities && data.capabilities.colorDepth) {
        if(data.capabilities.colorDepth === 4) {
            depth = "Grayscale";
        } else if(data.capabilities.colorDepth === 1) {
            depth = "Black and White";
        }
    }
    colorDepthStatus.appendChild(document.createTextNode(depth));
    supportedRegions.appendChild(document.createTextNode(data.regions.length));
    digitizerSize.appendChild(document.createTextNode(data.capabilities ? data.capabilities.digitizerWidth + "x" + data.capabilities.digitizerHeight : "N/A"));

    var aspect = "N/A";
    if(data.displayInfo) {
        var gcd = function(a, b) { return (!b) ? a : gcd (b, a % b); };
        var div = gcd(data.displayInfo.width, data.displayInfo.height);
        aspect = (data.displayInfo.width / div) + "x" + (data.displayInfo.height / div);
    }
    aspectRatio.appendChild(document.createTextNode(aspect));

    this.drawErrorCorrectionTable(data);

    currentlyCalibrating.appendChild(document.createTextNode(data.calibrationMode.calibrating ? "True" : "False"));
    calibrationPoint.appendChild(document.createTextNode(data.calibrationMode.calibrationPoint));

    this.loadFirmwareVersions(devInfo.vendorId, devInfo.productId);

    var firmwareTab = document.getElementById("device_tab_firmware");
    var displayTab = document.getElementById("device_tab_display");
    var cardReaderTab = document.getElementById("device_tab_cardreader");
    if(this.hasDisplay()) {
        firmwareTab.className = firmwareTab.className.replace(" device_tab_disabled", "");
        displayTab.className = displayTab.className.replace(" device_tab_disabled", "");
    } else {
        firmwareTab.className += " device_tab_disabled";
        displayTab.className += " device_tab_disabled";
    }

    if(this.deviceInfo.capabilities && this.deviceInfo.capabilities.magReaderId != "NONE") {
        cardReaderTab.className = displayTab.className.replace(" device_tab_disabled", "");
    } else {
        cardReaderTab.className += " device_tab_disabled";
    }

    var brightnessSlider = document.getElementById("device_display_brightness");
    var contrastSlider = document.getElementById("device_display_contrast");
    var brightnessLevels = (this.deviceInfo.capabilities && this.deviceInfo.capabilities.brightnessLevels) ? this.deviceInfo.capabilities.brightnessLevels : 100;
    var contrastLevels = (this.deviceInfo.capabilities && this.deviceInfo.capabilities.contrastLevels) ? this.deviceInfo.capabilities.contrastLevels : 100;

    if(brightnessLevels === 2) {
        brightnessLevels = 1;
    }

    if(contrastLevels === 2) {
        contrastLevels = 1;
    }

    if(brightnessLevels > 0) {
        brightnessSlider.enabled = true;
        brightnessSlider.setAttribute("max", brightnessLevels);
        brightnessSlider.setAttribute("step", (brightnessLevels > 50) ? 5 : 1);
        brightnessSlider.value = this.deviceInfo.displaySettings.brightness;
    } else {
        brightnessSlider.enabled = false;
        brightnessSlider.value = 50;
    }

    if(contrastLevels > 0) {
        contrastSlider.enabled = true;
        contrastSlider.setAttribute("max", contrastLevels);
        contrastSlider.setAttribute("step", (contrastLevels > 50) ? 5 : 1);
        contrastSlider.value = this.deviceInfo.displaySettings.contrast;
    } else {
        contrastSlider.enabled = false;
        contrastSlider.value = 50;
    }
};
ScriptelWebapp.prototype.hasDisplay = function() {
    "use strict";
    if(this.deviceInfo.capabilities && this.deviceInfo.capabilities.displayId != "NONE") {
        return true;
    } else if(this.deviceInfo.displayInfo && this.deviceInfo.displayInfo.width > 0 && this.deviceInfo.displayInfo.height > 0) {
        return true;
    } else if(this.deviceInfo.vendorId === 0x18fb && this.deviceInfo.productId === 0x0200) {
        return true;
    }
    return false;
};
ScriptelWebapp.prototype.drawErrorCorrectionTable = function (devInfo) {
    "use strict";
    var errCor = devInfo.errorCorrection;
    var canvas = document.getElementById("device_glass_error_plot");
    var ctx = canvas.getContext("2d");
    var i, j;

    var padding = 20;

    var scale = 2;
    var width = (devInfo.displayInfo.width * scale) + padding;
    var height = (devInfo.displayInfo.height * scale) + padding;
    canvas.width = width;
    canvas.height = height;

    ctx.fillStyle = "white";
    ctx.fillRect(0, 0, width, height);

    ctx.fillStyle = "black";
    ctx.strokeStyle = "black";

    var matrix = [];

    for (i = 0; i < 26; i++) {
        matrix[i] = [];
        for (j = 0; j < 26; j++) {
            var x = errCor.correctionMap[0][(i * 26) + j];
            var y = errCor.correctionMap[1][(i * 26) + j];

            x = (x - errCor.boundX1) / (errCor.boundX2 - errCor.boundX1);
            y = (y - errCor.boundY1) / (errCor.boundY2 - errCor.boundY1);

            x *= (width - padding);
            y *= (height - padding);

            x += padding / 2;
            y += padding / 2;

            matrix[i][j] = {"x": x, "y": y};
        }
    }

    for (i = 0; i < matrix.length; i++) {
        for (j = 0; j < matrix[i].length; j++) {
            if (j > 0) {
                ctx.beginPath();
                ctx.moveTo(matrix[i][j - 1].x, matrix[i][j - 1].y);
                ctx.lineTo(matrix[i][j].x, matrix[i][j].y);
                ctx.stroke();
            }

            if (i > 0) {
                ctx.beginPath();
                ctx.moveTo(matrix[i - 1][j].x, matrix[i - 1][j].y);
                ctx.lineTo(matrix[i][j].x, matrix[i][j].y);
                ctx.stroke();
            }

            ctx.beginPath();
            ctx.arc(matrix[i][j].x, matrix[i][j].y, 1, 0, Math.PI * 2);
            ctx.fill();
        }
    }
};

ScriptelWebapp.prototype.empty = function (element) {
    "use strict";
    while (element.firstChild) {
        element.removeChild(element.firstChild);
    }
};

ScriptelWebapp.prototype.loadFirmwareVersions = function(vendorId, productId) {
    var req = new XMLHttpRequest();
    var baseUrl = "https://services.scriptel.com/releases/firmware/";
    var fileName = vendorId + "_" + productId;
    var currentVersion = document.getElementById("device_firmware_current_version");
    var latestVersion = document.getElementById("device_firmware_latest_version");
    var versions = document.getElementById("device_firmware_internet_version");
    var dlButton = document.getElementById("firmware_download_internet_button");

    var latestUpdated = false;
    var t = this;

    req.onreadystatechange = function() {
        if(req.readyState === 4 && req.status === 200) {
            versions.options.length = 0;
            var data = JSON.parse(req.responseText);
            var i, j;
            for(i = 0; i < data.releases.length; i++) {
                var release = data.releases[i];
                for(j = 0; j < release.files.length; j++) {
                    var file = release.files[j];
                    if(file.name.toLowerCase() === fileName.toLowerCase()) {
                        var name = "Release " + release.versionMajor + "." + release.versionMinor + "." + release.versionRelease + " (" + release.date.substring(0, 10) + ")";
                        var url = baseUrl + fileName + "/" + release.id;
                        versions.options[versions.options.length] = new Option(name, url);

                        if(!latestUpdated) {
                            latestVersion.appendChild(document.createTextNode(release.versionMajor + "." + release.versionMinor + "." + release.versionRelease));
                            
                            var cmp = t.firmwareVersionCompare(t.deviceInfo.softwareVersion.firmwareVersion.major, t.deviceInfo.softwareVersion.firmwareVersion.minor, t.deviceInfo.softwareVersion.firmwareVersion.release, release.versionMajor, release.versionMinor, release.versionRelease);
                            if(cmp < 0) {
                                //Firmware version is older than newest.
                                var div = document.createElement("div");
                                div.className = "device_firmware_update_available";
                                div.appendChild(document.createTextNode("Update Available!"));
                                currentVersion.appendChild(div);
                            }

                            latestUpdated = true;
                        }
                    }
                }
            }

            if(versions.options.length !== 0) {
                versions.disabled = false;
                dlButton.disabled = false;
            }
        }
    }
    req.open("GET", baseUrl, true);
    req.send();
};

ScriptelWebapp.prototype.getInternetFirmware = function(url) {
    "use strict";
    var t = this;
    if (confirm("Are you sure you want to update the firmware on this device?\nPlease do not unplug the device while firmware is updating.")) {
        var req = new XMLHttpRequest();
        req.open("GET", url, true);
        req.responseType = "arraybuffer";    
        req.onload = function(evt) {
            var buffer = req.response;
            if(buffer) {
                var dataUrl = t.arrBufferToDataUrl("application/x-gzip", buffer);
                t.firmwareUpdate = { "_class": "FirmwareUpdateRequest", "update": dataUrl };
                t.api.uploadFirmware(t.firmwareUpdate);
            }
        }

        req.send(); 
    }
    
};

ScriptelWebapp.prototype.arrBufferToDataUrl = function(mimeType, buffer) {
    "use strict";
    var result = "";
    var arr = new Uint8Array(buffer);
    var len = arr.byteLength;
    var i;
    for(i = 0; i < len; i++) {
        result += String.fromCharCode(arr[i]);
    }
    return "data:" + mimeType + ";base64," + btoa(result);
};

ScriptelWebapp.prototype.firmwareVersionCompare = function(v1Major, v1Minor, v1Release, v2Major, v2Minor, v2Release) {
    v1Major *= 1;
    v1Minor *= 1;
    v1Release *= 1;
    v2Major *= 1;
    v2Minor *= 1;
    v2Release *= 1;

    if(v1Major < v2Major) {
        return -1;
    } else if(v1Major > v2Major) {
        return 1;
    } else {
        if(v1Minor < v2Minor) {
            return -1;
        } else if(v1Minor > v2Minor) {
            return 1;
        } else {
            if(v1Release < v2Release) {
                return -1;
            } else if(v1Release > v2Release) {
                return 1;
            } else {
                return 0;
            }
        }
    }
};

ScriptelWebapp.prototype.drawRegionTree = function() {
    var el = document.getElementById("device_region_tree");
    this.empty(el);
    var tree = this.buildRegionTree();
    var i;
    var t = this;
    for(var id in tree) {
        if(tree.hasOwnProperty(id)) {
            var item = document.createElement("div");
            item.className = "device_region_tree_parent";
            
            var subItem = document.createElement("div");
            subItem.className = "device_region_tree_parent_item device_region_tree_item";
            subItem.appendChild(document.createTextNode(id + " - " + this.getRegionTypeLabel(this.getRegionType(id))));
            item.appendChild(subItem);
            subItem.setAttribute("data-region-id", id);
            subItem.id = "device_region_tree_item_" + id;

            subItem.addEventListener("click", function(evt) {
                t.selectTreeRegion(this.getAttribute("data-region-id") * 1);
            }, false);

            el.appendChild(item);

            var treeItem = tree[id];

            var j;
            for(j = 0; j < treeItem.children.length; j++) {
                var childId = treeItem.children[j];
                var childItem = document.createElement("div");
                childItem.className = "device_region_tree_child device_region_tree_item";
                childItem.appendChild(document.createTextNode(childId + " - " + this.getRegionTypeLabel(this.getRegionType(childId))));
                childItem.setAttribute("data-region-id", childId);
                childItem.id = "device_region_tree_item_" + childId;
                item.appendChild(childItem);

                childItem.addEventListener("click", function(evt) {
                    t.selectTreeRegion(this.getAttribute("data-region-id") * 1);
                }, false);
            }
        }
    }
    this.updateActiveContainerMarker();
};

ScriptelWebapp.prototype.selectTreeRegion = function(id) {
    "use strict";
    
    var elList = document.getElementsByClassName("device_region_tree_selected");
    var i;
    for(i = 0; i < elList.length; i++) {
        elList[i].className = elList[i].className.replace(" device_region_tree_selected", "");
    }

    if(id >= 1) {
        var el = document.getElementById("device_region_tree_item_" + id);
        el.className += " device_region_tree_selected";
        this.selectRegion(id);

        var warning = document.getElementById("device_region_editor_warning");
        if(id == 1 || id == this.deviceInfo.displayInfo.regionCount) {
            warning.style.display = "block";
        } else {
            warning.style.display = "none";
        }
    } else {
        this.clearEditorElements();
        this.disableEditorElements(true);
    }
};

ScriptelWebapp.prototype.getEditorElements = function() {
    "use strict";

    if(!this.editorElements) {
        var elements = {};
        elements.number = document.getElementById("device_region_number");
        elements.type = document.getElementById("device_region_type");
        elements.x1 = document.getElementById("device_region_x1");
        elements.y1 = document.getElementById("device_region_y1");
        elements.x2 = document.getElementById("device_region_x2");
        elements.y2 = document.getElementById("device_region_y2");
        elements.fgColor = document.getElementById("device_region_fg_color");
        elements.bgColor = document.getElementById("device_region_bg_color");
        elements.fgPreview = document.getElementById("device_region_fg_color_preview");
        elements.bgPreview = document.getElementById("device_region_bg_color_preview");
        elements.acceptsTouch = document.getElementById("device_region_accepts_touch");
        elements.decode = document.getElementById("device_region_decode");
        elements.visible = document.getElementById("device_region_visible");
        elements.inUse = document.getElementById("device_region_in_use");
        elements.hasFrame = document.getElementById("device_region_has_frame");
        elements.buttonFlags = document.getElementById("device_region_editor_flags_button");
        elements.lineFlags = document.getElementById("device_region_editor_flags_line");
        elements.containerFlags = document.getElementById("device_region_editor_flags_container");
        elements.buttonProps = document.getElementById("device_region_editor_props_button");
        elements.lineProps = document.getElementById("device_region_editor_props_line");
        elements.containerProps = document.getElementById("device_region_editor_props_container");
        elements.inkEnabled = document.getElementById("device_region_container_ink_enabled");
        elements.buttonCaption = document.getElementById("device_region_button_caption");
        elements.buttonParent = document.getElementById("device_region_button_parent");
        elements.buttonEnabled = document.getElementById("device_region_button_enabled");
        elements.buttonHighlightOnSelect = document.getElementById("device_region_button_highlight");
        elements.buttonClearOnSelect = document.getElementById("device_region_button_clear");
        elements.lineWidth = document.getElementById("device_region_line_width");
        elements.lineParent = document.getElementById("device_region_line_parent");
        elements.regionWidth = document.getElementById("device_region_width");
        elements.regionHeight = document.getElementById("device_region_height");

        var t = this;
        var onchange = function() {
            "use strict";
            var region = t.getEditorRegion();
            if(region) {
                t.deviceInfo.regions[region.id - 1] = region;
                t.display.setRegion(region);
                t.drawRegionTree();
                t.selectTreeRegion(region.id);
                t.api.updateRegion(region);
            }
        };

        elements.type.addEventListener("change", function() {
            //We need to fill in some reasonable defaults to make things less wonky.
            if(elements.type.value === "button") {
                elements.buttonCaption.value = "Button";
                elements.fgColor.value = "#000000ff";
                elements.bgColor.value = "#ffffffff";
                elements.acceptsTouch.checked = true;
                elements.decode.checked = true;
                elements.visible.checked = true;
                elements.inUse.checked = true;
                elements.hasFrame.checked = true;
                elements.buttonEnabled.checked = true;
                elements.buttonHighlightOnSelect.checked = true;
                elements.buttonClearOnSelect.checked = true;

                var x1 = parseInt(elements.x1.value);
                var x2 = parseInt(elements.x2.value);
                var y1 = parseInt(elements.y1.value);
                var y2 = parseInt(elements.y2.value);

                var tmp;
                if(x1 > x2) {
                    tmp = x1;
                    x1 = x2;
                    x2 = tmp;
                }

                if(y1 > y2) {
                    tmp = y1;
                    y1 = y2;
                    y2 = tmp;
                }

                if(x2 - x1 < 40) {
                    x2 = x1 + 40;
                }

                if(y2 - y1 < 12) {
                    y2 = y1 + 12;
                }

                elements.x1.value = x1;
                elements.x2.value = x2;
                elements.y1.value = y1;
                elements.y2.value = y2;

                if(t.display.activeContainer > 0) {
                    elements.buttonParent.options[elements.buttonParent.options.length] = new Option(t.display.activeContainer, t.display.activeContainer);
                    elements.buttonParent.value = t.display.activeContainer;
                }
            } else if(elements.type.value === "line") {
                elements.acceptsTouch.checked = false;
                elements.decode.checked = false;
                elements.visible.checked = true;
                elements.inUse.checked = true;
                elements.hasFrame.checked = false;

                elements.lineWidth.value = 1;

                elements.fgColor.value = "#000000ff";
                elements.bgColor.value = "#ffffffff";

                var x1 = parseInt(elements.x1.value);
                var x2 = parseInt(elements.x2.value);
                var y1 = parseInt(elements.y1.value);
                var y2 = parseInt(elements.y2.value);

                var tmp;
                if(x1 > x2) {
                    tmp = x1;
                    x1 = x2;
                    x2 = tmp;
                }

                if(y1 > y2) {
                    tmp = y1;
                    y1 = y2;
                    y2 = tmp;
                }

                if(x2 - x1 < 20) {
                    x2 = x1 + 20;
                }

                if(y2 - y1 < 20) {
                    y2 = y1 + 20;
                }

                elements.x1.value = x1;
                elements.x2.value = x2;
                elements.y1.value = y1;
                elements.y2.value = y2;

                if(t.display.activeContainer > 0) {
                    elements.lineParent.options[elements.lineParent.options.length] = new Option(t.display.activeContainer, t.display.activeContainer);
                    elements.lineParent.value = t.display.activeContainer;
                }
            } else if(elements.type.value === "container") {
                elements.acceptsTouch.checked = true;
                elements.decode.checked = true;
                elements.visible.checked = true;
                elements.inUse.checked = true;
                elements.hasFrame.checked = false;
                elements.x1.value = 0;
                elements.y1.value = 0;
                elements.x2.value = t.deviceInfo.displayInfo.width - 1;
                elements.y2.value = t.deviceInfo.displayInfo.height - 1;
                elements.fgColor.value = "#000000ff";
                elements.bgColor.value = "#ffffffff";
            } else if(elements.type.value === "null") {
                elements.x1.value = 0;
                elements.y1.value = 0;
                elements.x2.value = 0;
                elements.y2.value = 0;
                elements.fgColor.value = "#00000000";
                elements.bgColor.value = "#00000000";
                elements.acceptsTouch.checked = false;
                elements.decode.checked = false;
                elements.visible.checked = false;
                elements.inUse.checked = false;
                elements.hasFrame.checked = false;
            }
        }, false);

        elements.type.addEventListener("change", onchange, false);
        elements.x1.addEventListener("change", onchange, false);
        elements.y1.addEventListener("change", onchange, false);
        elements.x2.addEventListener("change", onchange, false);
        elements.y2.addEventListener("change", onchange, false);
        elements.fgColor.addEventListener("change", onchange, false);
        elements.bgColor.addEventListener("change", onchange, false);
        elements.acceptsTouch.addEventListener("change", onchange, false);
        elements.decode.addEventListener("change", onchange, false);
        elements.visible.addEventListener("change", onchange, false);
        elements.inUse.addEventListener("change", onchange, false);
        elements.hasFrame.addEventListener("change", onchange, false);
        elements.inkEnabled.addEventListener("change", onchange, false);
        elements.buttonCaption.addEventListener("change", onchange, false);
        elements.buttonParent.addEventListener("change", onchange, false);
        elements.buttonEnabled.addEventListener("change", onchange, false);
        elements.buttonHighlightOnSelect.addEventListener("change", onchange, false);
        elements.buttonClearOnSelect.addEventListener("change", onchange, false);
        elements.lineWidth.addEventListener("change", onchange, false);
        elements.lineParent.addEventListener("change", onchange, false);

        this.editorElements = elements;
    }
    return this.editorElements;
};

ScriptelWebapp.prototype.clearEditorElements = function() {
    "use strict";
    var elements = this.getEditorElements();
    elements.number.value = "";
    elements.type.value = "null";
    elements.x1.value = "";
    elements.y1.value = "";
    elements.x2.value = "";
    elements.y2.value = "";
    elements.regionWidth.value = "";
    elements.regionHeight.value = "";
    elements.fgColor.value = "";
    elements.bgColor.value = "";
    elements.fgPreview.style.backgroundColor = "#FFFFFF";
    elements.bgPreview.style.backgroundColor = "#FFFFFF";

    elements.acceptsTouch.checked = false;
    elements.decode.checked = false;
    elements.visible.checked = false;
    elements.inUse.checked = false;
    elements.hasFrame.checked = false;

    elements.containerFlags.style.display = "none";
    elements.containerProps.style.display = "none";
    elements.buttonFlags.style.display = "none";
    elements.buttonProps.style.display = "none";
    elements.lineFlags.style.display = "none";
    elements.lineProps.style.display = "none";

    elements.inkEnabled.checked = false;
    elements.buttonEnabled.checked = false;
    elements.buttonHighlightOnSelect.checked = false;
    elements.buttonClearOnSelect.checked = false;

    elements.lineWidth.value = "";
    elements.buttonCaption.value = "";
    elements.lineParent.options.length = 0;
    elements.buttonParent.options.length = 0;
};

ScriptelWebapp.prototype.disableEditorElements = function(disable) {
    "use strict";
    var elements = this.getEditorElements();
    elements.type.disabled = disable;
    elements.x1.disabled = disable;
    elements.y1.disabled = disable;
    elements.x2.disabled = disable;
    elements.y2.disabled = disable;
    elements.fgColor.disabled = disable;
    elements.bgColor.disabled = disable;

    elements.acceptsTouch.disabled = disable;
    elements.decode.disabled = disable;
    elements.visible.disabled = disable;
    elements.inUse.disabled = disable;
    elements.hasFrame.disabled = disable;

    elements.inkEnabled.disabled = disable;
    elements.buttonEnabled.disabled = disable;
    elements.buttonHighlightOnSelect.disabled = disable;
    elements.buttonClearOnSelect.disabled = disable;

    elements.lineWidth.disabled = disable;
    elements.buttonCaption.disabled = disable;
    elements.lineParent.disabled = disable;
    elements.buttonParent.disabled = disable;
};

ScriptelWebapp.prototype.getEditorRegion = function() {
    "use strict";
    var elements = this.getEditorElements();
    var id = parseInt(elements.number.value);

    if(id <= 0) {
        return false;
    }

    var type = elements.type.value;
    var region = {
        _class: "Region",
        id: id,
        x1: parseInt(elements.x1.value),
        y1: parseInt(elements.y1.value),
        x2: parseInt(elements.x2.value),
        y2: parseInt(elements.y2.value),
        foregroundColor: this.convertColorToArr(elements.fgColor.value),
        backgroundColor: this.convertColorToArr(elements.bgColor.value),
        flags: {
            _class: "RegionFlags",
            acceptsTouch: elements.acceptsTouch.checked,
            decodeRegion: elements.decode.checked,
            hasFrame: elements.hasFrame.checked,
            inUse: elements.inUse.checked,
            visible: elements.visible.checked
        }
    };

    if(type === "container") {
        region.containerType = {
            _class: "RegionContainer",
            flags: {
                _class: "RegionContainerFlags",
                inkEnabled: elements.inkEnabled.checked
            }
        };
    } else if(type === "button") {
        region.buttonType = {
            _class: "RegionButton",
            flags: {
                _class: "RegionButtonFlags",
                clearOnSelect: elements.buttonClearOnSelect.checked,
                highlightOnSelect: elements.buttonHighlightOnSelect.checked,
                enabled: elements.buttonEnabled.checked
            },
            caption: elements.buttonCaption.value,
            parentContainer: parseInt(elements.buttonParent.value)
        };
    } else if(type === "line") {
        region.lineType = {
            _class: "RegionLine",
            lineWidth: parseInt(elements.lineWidth.value),
            parentContainer: parseInt(elements.lineParent.value)
        };
    }

    return region;
};

ScriptelWebapp.prototype.selectRegion = function(id) {
    "use strict";

    this.disableEditorElements(false);

    var region = this.deviceInfo.regions[id - 1];
    this.display.selectedRegion = id - 1;
    this.display.setRegion(region);

    var elements = this.getEditorElements();

    elements.lineParent.options.length = 0;
    elements.buttonParent.options.length = 0;

    var i;
    for(i = 0; i < this.deviceInfo.regions.length; i++) {
        var r = this.deviceInfo.regions[i];
        if(r.containerType) {
            elements.lineParent.options[elements.lineParent.options.length] = new Option(r.id, r.id);
            elements.buttonParent.options[elements.buttonParent.options.length] = new Option(r.id, r.id);
        }
    }

    elements.number.value = region.id;
    elements.type.value = this.getRegionType(id);
    elements.x1.value = region.x1;
    elements.y1.value = region.y1;
    elements.x2.value = region.x2;
    elements.y2.value = region.y2;
    elements.regionWidth.value = Math.abs(region.x2 - region.x1);
    elements.regionHeight.value = Math.abs(region.y2 - region.y1);
    elements.fgColor.value = this.convertColorFromArr(region.foregroundColor);
    elements.bgColor.value = this.convertColorFromArr(region.backgroundColor);
    elements.fgPreview.style.backgroundColor = elements.fgColor.value.substring(0, 7);
    elements.bgPreview.style.backgroundColor = elements.bgColor.value.substring(0, 7);

    elements.acceptsTouch.checked = region.flags.acceptsTouch;
    elements.decode.checked = region.flags.decodeRegion;
    elements.visible.checked = region.flags.visible;
    elements.inUse.checked = region.flags.inUse;
    elements.hasFrame.checked = region.flags.hasFrame;

    if(region.containerType) {
        elements.containerFlags.style.display = "block";
        elements.containerProps.style.display = "block";
        elements.buttonFlags.style.display = "none";
        elements.buttonProps.style.display = "none";
        elements.lineFlags.style.display = "none";
        elements.lineProps.style.display = "none";

        elements.inkEnabled.checked = region.containerType.flags.inkEnabled;
    } else if(region.buttonType) {
        elements.containerFlags.style.display = "none";
        elements.containerProps.style.display = "none";
        elements.buttonFlags.style.display = "block";
        elements.buttonProps.style.display = "block";
        elements.lineFlags.style.display = "none";
        elements.lineProps.style.display = "none";

        elements.buttonCaption.value = region.buttonType.caption;
        elements.buttonEnabled.checked = region.buttonType.flags.enabled;
        elements.buttonHighlightOnSelect.checked = region.buttonType.flags.highlightOnSelect;
        elements.buttonClearOnSelect.checked = region.buttonType.flags.clearOnSelect;
        elements.buttonParent.value = region.buttonType.parentContainer;
    } else if(region.lineType) {
        elements.containerFlags.style.display = "none";
        elements.containerProps.style.display = "none";
        elements.buttonFlags.style.display = "none";
        elements.buttonProps.style.display = "none";
        elements.lineFlags.style.display = "block";
        elements.lineProps.style.display = "block";

        elements.lineWidth.value = region.lineType.lineWidth;
        elements.lineParent.value = region.lineType.parentContainer;
    } else {
        elements.containerFlags.style.display = "none";
        elements.containerProps.style.display = "none";
        elements.buttonFlags.style.display = "none";
        elements.buttonProps.style.display = "none";
        elements.lineFlags.style.display = "none";
        elements.lineProps.style.display = "none";
    }
};

ScriptelWebapp.prototype.convertColorFromArr = function(color) {
    "use strict";

    if(!color || !Array.isArray(color)) {
        return "#000000ff";
    }

    var r = color[0].toString(16);
    var g = color[1].toString(16);
    var b = color[2].toString(16);
    var a = color[3].toString(16);

    while(r.length != 2) {
        r = "0" + r;
    }
    while(g.length != 2) {
        g = "0" + g;
    }
    while(b.length != 2) {
        b = "0" + b;
    }
    while(a.length != 2) {
        a = "0" + a;
    }
    return "#" + r + g + b + a;
};

ScriptelWebapp.prototype.convertColorToArr = function(color) {
    "use strict";

    if(color.charAt(0) != "#" || color.length != 9) {
        return [0, 0, 0, 0xFF];
    }

    var r = parseInt(color.substring(1, 3), 16);
    var g = parseInt(color.substring(3, 5), 16);
    var b = parseInt(color.substring(5, 7), 16);
    var a = parseInt(color.substring(7, 9), 16);

    return [r, g, b, a];
};

ScriptelWebapp.prototype.getRegionTypeLabel = function(type) {
    switch(type) {
        case "container":
            return "Container";
        case "line":
            return "Line";
        case "button":
            return "Button";
        case "null":
            return "Unused";
    }
    return "Unknown";
}

ScriptelWebapp.prototype.getRegionType = function(id) {
    if(this.deviceInfo.regions && this.deviceInfo.regions[id - 1]) {
        var region = this.deviceInfo.regions[id - 1];
        if(region.containerType) {
            return "container";
        } else if(region.buttonType) {
            return "button";
        } else if(region.lineType) {
            return "line";
        } else {
            return "null";
        }
    }
    return "unknown";
};

ScriptelWebapp.prototype.buildRegionTree = function() {
    var tree = {};
    var i;

    //We'll do two passes, one for everything but lines and
    //buttons, and then another to those.
    for(i = 0; i < this.deviceInfo.regions.length; i++) {
        var region = this.deviceInfo.regions[i];
        if(!region.lineType && !region.buttonType) {
            tree[region.id] = {"type": (region.containerType) ? "container" : "null", "children": []};
        }
    }

    for(i = 0; i < this.deviceInfo.regions.length; i++) {
        var region = this.deviceInfo.regions[i];
        if(region.lineType || region.buttonType) {
            var parent = region.lineType ? region.lineType.parentContainer : region.buttonType.parentContainer;

            if(tree[parent]) {
                tree[parent].children[tree[parent].children.length] = region.id;
            } else {
                tree[region.id] = {"type": (region.lineType) ? "line" : "button", "children": []};
            }
        }
    }

    return tree;
};
