/*
 * scriptel-proscript.js - Copyright 2015 Scriptel Corporation
 * ----------------
 * This file doesn't actually contain any logic, it exists as a documentation
 * and IDE reference for what's contained in the native library.
 */

/**
 * This class allows you to enumerate and listen for ScripTouch devices attached
 * to the system.
 * @constructor
 * @class
 */
function DeviceManager() {
    "use strict";
}
/**
 * This method returns a list of all devices currently attached to the system.
 * @returns {Array<Device>} List of devices attached to the system.
 */
DeviceManager.prototype.getAttachedDevices = function() {
    "use strict";
};

/**
 * This callback is designed to get notifications about system device membership
 * changes. 
 * @callback DeviceManager~deviceCallback
 * @param {DeviceMembershipEvent} event A device membership event.
 */

/**
 * This class contains information about a device membership event.
 * @constructor
 * @class
 */
function DeviceMembershipEvent() {
    "use strict";
    /**
     * The type of event, either "ATTACHED", "DETACHED", or "UNKNOWN".
     * @type {string}
     */
    this.type = "";
    /**
     * The path of the device.
     * @type {string}
     */
    this.path = "";
}
/**
 * This method adds a listener interested in device membership events.
 * @param {DeviceManager~deviceCallback} listener Listener that should get notifications.
 */
DeviceManager.prototype.addDeviceListener = function(listener) {
    "use strict";
};
/**
 * This method removes a listener no longer interested in device membership events.
 * @param {DeviceManager~deviceCallback} listener Listener that should not get notifications.
 */
DeviceManager.prototype.removeDeviceListener = function(listener) {
    "use strict";
};
/**
 * This function gets the wrapper version as a string. The wrapper is the glue
 * between the native ProScript library and node.js.
 * @returns {string} Version number of the wrapper.
 */
DeviceManager.prototype.getWrapperVersion = function() {
    "use strict";
};
/**
 * This function gets the major version of the wrapper.
 * @returns {number} The major version of the wrapper.
 */
DeviceManager.prototype.getWrapperVersionMajor = function() {
    "use strict";
};
/**
 * This function gets the minor version of the wrapper.
 * @returns {number} The minor version of the wrapper.
 */
DeviceManager.prototype.getWrapperVersionMinor = function() {
    "use strict";
};
/**
 * This function gets the release version of the wrapper.
 * @returns {number} The release version of the wrapper.
 */
DeviceManager.prototype.getWrapperVersionRelease = function() {
    "use strict";
};
/**
 * This function gets the build date of the wrapper in the format:
 * yyyy-MM-dd HH:mm:ssZ
 * @returns {string} Build date of the wrapper.
 */
DeviceManager.prototype.getWrapperBuildDate = function() {
    "use strict";
};
/**
 * This function gets the version of the native ProScript library being used.
 * @returns {string} The version of the native ProScript library being used.
 */
DeviceManager.prototype.getLibraryVersion = function() {
    "use strict";
};
/**
 * This function gets the major version of the native ProScript library.
 * @returns {number} The major version of the native ProScript library.
 */
DeviceManager.prototype.getLibraryVersionMajor = function() {
    "use strict";
};
/**
 * This function gets the minor version of the native ProScript library.
 * @returns {number} The minor version of the native ProScript library.
 */
DeviceManager.prototype.getLibraryVersionMinor = function() {
    "use strict";
};
/**
 * This function gets the release version of the native ProScript library.
 * @returns {number} The release version of the native ProScript library.
 */
DeviceManager.prototype.getLibraryVersionRelease = function() {
    "use strict";
};
/**
 * This function gets the build date of the native ProScript library in the format:
 * yyyy-MM-dd HH:mm:ssZ
 * @returns {string} Build date of the native ProScript library.
 */
DeviceManager.prototype.getLibraryBuildDate = function() {
    "use strict";
};

/**
 * This class represents a ScripTouch device attached to the system.
 * @constructor
 * @class
 */
function Device() {
    "use strict";
    /**
     * The local system path associated with this device.
     * @type {string}
     */
    this.path = "";
    /**
     * The model name of the device as reported by the device.
     * @type {string}
     */
    this.model = "";
    /**
     * The manufacturer of the device as reported by the device.
     * @type {string}
     */
    this.manufacturer = "";
    /**
     * The USB vendor ID of the device in BCD format.
     * @type {number}
     */
    this.vendorId = 0;
    /**
     * The USB product ID of the device in BCD format.
     */
    this.productId = 0;
}

/**
 * This function attempts to open the device.
 * @throws Will throw an exception if the device can't be opened.
 */
Device.prototype.open = function() {
    "use strict";
};
/**
 * This function attempts to close the device.
 */
Device.prototype.close = function() {
    "use strict";
};
/**
 * This function gets the UUID of the device as an array containing 16 elements.
 * @throws Will throw an exception if the uuid can't be read.
 * @returns {Array<number>} Array of 16 elements containing the uuid.
 */
Device.prototype.getUuid = function() {
    "use strict";
};
/**
 * This function gets the UUID of the device as a string.
 * @throws Will throw an exception if the uuid can't be read.
 * @returns {string} The uuid of the device as a string.
 */
Device.prototype.getUuidString = function() {
    "use strict";
};
/**
 * This function gets whether or not the device is currently open.
 * @returns {boolean} Whether or not the device is currently open.
 */
Device.prototype.isOpen = function() {
    "use strict";
};
/**
 * This function gets whether or not a button is currently depressed.
 * @returns {boolean} Whether or not a button is currently depressed.
 */
Device.prototype.isButtonDown = function() {
    "use strict";
};

/**
 * This callback is designed to get notifications about system device membership
 * changes. 
 * @callback Device~inputCallback
 * @param {DebugCoordinateEvent | KeyboardEvent | DigitizerTouchCoordinateEvent | DigitizerPenCoordinateEvent | MouseCoordinateEvent | AdcValuesEvent | ButtonPressEvent | ButtonUpEvent | ButtonMoveEvent | ButtonDownEvent | MagCardDataEvent | ExtendedInputCoordinateEvent | InputCoordinateEvent} event The input event captured from the device.
 */

/**
 * Registers an input callback to be notified when input is captured from the
 * device.
 * @param {Device~inputCallback} listener The listener to be notified.
 */
Device.prototype.addInputListener = function(listener) {
    "use strict";
};
/**
 * Unregisters an input callback to be notified when input is captured from the
 * device.
 * @param {Device~inputCallback} listener The listener to be removed.
 */
Device.prototype.removeInputListener = function(listener) {
    "use strict";
};
/**
 * This function gets the capabilities of a device from the device.
 * @returns {Capabilities} List of capabilities of the device.
 */
Device.prototype.getCapabilities = function() {
    "use strict";
};
/**
 * This function gets information about the display from the device.
 * @returns {DisplayInfo} Display info from the device.
 */
Device.prototype.getDisplayInfo = function() {
    "use strict";
};
/**
 * This function gets information about the current display configuration from
 * the device.
 * @returns {DisplaySettings} Display settings from the device.
 */
Device.prototype.getDisplaySettings = function() {
    "use strict";
};
/**
 * This function gets the current coordinate range from the device.
 * @returns {CoordinateRange} Coordinate range from the device.
 */
Device.prototype.getCoordinateRange = function() {
    "use strict";
};
/**
 * This function gets the firmware version information from the device.
 * @returns {VersionInfo} The firmware version information.
 */
Device.prototype.getVersionInfo = function() {
    "use strict";
};
/**
 * This function gets the regions in the device.
 * @returns {Array<LegacyRegion>} List of regions from the device.
 */
Device.prototype.getRegions = function() {
    "use strict";
};
/**
 * This function will grab the raw error correction table as an array of 2048
 * elements.
 * @returns {Array<number>} Error correction table as an array of 2048 elements.
 */
Device.prototype.getErrorCorrectionTable = function() {
    "use strict";
};
/**
 * This function gets the serial number from the device if set.
 * @returns {string} Serial number from the device.
 */
Device.prototype.getSerialNumber = function() {
    "use strict";
};
/**
 * This function gets the current power state of the onboard ASIC. Can be:
 * NORMAL, POWERDOWN, STANDBY, or UNKNOWN.
 * @returns {string} Current power state of the onboard ASIC.
 */
Device.prototype.getPowerState = function() {
    "use strict";
};
/**
 * This function gets the supported and current operating modes of the current
 * device.
 * @returns {OperatingModes} Operating modes of the device.
 */
Device.prototype.getOperatingModes = function() {
    "use strict";
};
/**
 * This function gets the general input parameters for the device.
 * @returns {GeneralParameters} General input parameters from the device.
 */
Device.prototype.getGeneralParameters = function() {
    "use strict";
};
/**
 * This function gets the frequency amplitude settings for the device.
 * @returns {FrequencyAmplitude} The current frequency and amplitude settings.
 */
Device.prototype.getFrequencyAmplitude = function() {
    "use strict";
};
/**
 * This function gets the ASIC registers.
 * @returns {Array<number>} Current ASIC registers.
 */
Device.prototype.getAsicRegister = function() {
    "use strict";
};
/**
 * This function gets the current pen parameters.
 * @returns {PenParameters} The current pen parameters.
 */
Device.prototype.getPenParameters = function() {
    "use strict";
};
/**
 * This function gets the current output configuration for the device.
 * @returns {OutputConfiguration} The current output configuration.
 */
Device.prototype.getOutputConfiguration = function() {
    "use strict";
};
/**
 * This function gets the current calibration data for the device.
 * @returns {CalibrationData} Calibration data from the device.
 */
Device.prototype.getCalibrationData = function() {
    "use strict";
};
/**
 * This function gets the current device status.
 * @returns {DeviceStatus} The current device status.
 */
Device.prototype.getDeviceStatus = function() {
    "use strict";
};
/**
 * This function gets the current touch parameters from the device.
 * @returns {TouchParameters} The current touch parameters.
 */
Device.prototype.getTouchParameters = function() {
    "use strict";
};
/**
 * This function gets the current calibration state.
 * @returns {CalibrationState} The current calibration state.
 */
Device.prototype.getCalibrationState = function() {
    "use strict";
};
/**
 * This function gets the maximum number of contacts that can be tracked
 * @returns {number} Contact count maximum.
 */
Device.prototype.getContactCountMaximum = function() {
    "use strict";
};
/**
 * Gets the current configuration from the device.
 * @returns {Configuration}
 */
Device.prototype.getConfiguration = function() {
    "use strict";
};
/**
 * Gets the default regions from the device.
 * @returns {Array<LegacyRegion>} Default regions from the device.
 */
Device.prototype.getDefaultRegions = function() {
    "use strict";
};
/**
 * Gets the driver name being used to service the current device.
 * @returns {string} Driver name.
 */
Device.prototype.getDriverName = function() {
    "use strict";
};

/**
 * Sets the output configuration options in a device.
 * @param outputConfig {OutputConfiguration} The output configuration to write.
 */
Device.prototype.setOutputConfiguration = function(outputConfig) {
    "use strict";
};
/**
 * Sets the calibration data in the device.
 * @param {CalibrationData} calData The calibration data to write.
 */
Device.prototype.setCalibrationData = function(calData) {
    "use strict";
};
/**
 * Sets the current run mode of the device, can be: COMMAND, TOUCH, PEN,
 * AUTOTOUCH, TOUCHSINGLE, TOUCHSTREAM, PENSINGLE, PENSTREAM, PENCONTINIOUS
 * @param {string} runMode The run mode to set.
 */
Device.prototype.setRunMode = function(runMode) {
    "use strict";
};

/**
 * Sets the serial number in the device.
 * @param {string} serial The serial number to write.
 */
Device.prototype.setSerialNumber = function(serial) {
    "use strict";
};
/**
 * This method attempts to clear the screen of the device.
 */
Device.prototype.clearScreen = function() {
    "use strict";
};

/**
 * This method attempts to delete a region.
 * @param {number} Region number to delete (1-indexed).
 */
Device.prototype.deleteRegion = function(regionNumber) {
    "use strict";
};
/**
 * This method sets the display settings in a device.
 * @param {DisplaySettings} displaySettings Display settings to write.
 */
Device.prototype.setDisplaySettings = function(displaySettings) {
    "use strict";
};
/**
 * Sets the configuration in a device.
 * @param {Configuration} config Configuration to write.
 */
Device.prototype.setDeviceConfiguration = function(config) {
    "use strict";
};

/**
 * Sets the frequency and amplitude settings in a device.
 * @param {FrequencyAmplitude} freqAmp Freqency and amplitude settings to write.
 */
Device.prototype.setFrequencyAmplitude = function(freqAmp) {
    "use strict";
};
/**
 * Sets the power state in a device, can be: NORMAL, POWERDOWN, STANDBY
 * @param {string} powerState The power state to set.
 */
Device.prototype.setPowerState = function(powerState) {
    "use strict";
};
/**
 * This sets a firmware page in the device.
 * @param {FirmwarePage} page Page to write to the device.
 */
Device.prototype.setFirmwarePage = function(page) {
    "use strict";
};
/**
 * This function tells the device the firmware update is complete and that a
 * restart should be attempted.
 */
Device.prototype.setFirmwareComplete = function() {
    "use strict";
};
/**
 * This function sets the error correction table in the device.
 * @param {Array<number>} ect Error correction table to burn in, should be 2048 numeric elements.
 */
Device.prototype.setErrorCorrectionTable = function(ect) {
    "use strict";
};
/**
 * This function sets the startup mode of the device. Should be one of these:
 * TOUCH, PEN, AUTOTOUCH, COMMAND
 * @param mode Mode to set.
 */
Device.prototype.setStartupMode = function(mode) {
    "use strict";
};
/**
 * This function writes general parameters to a device.
 * @param {GeneralParameters} generalParams General parameters to write.
 */
Device.prototype.setGeneralParameters = function(generalParams) {
    "use strict";
};
/**
 * This function performs a reset of the onboard ASIC.
 * @param {boolean} hardReset True for a hardware reset, false for software.
 */
Device.prototype.asicReset = function(hardReset) {
    "use strict";
};
/**
 * This function sets a single ASIC register.
 * @param {AsicRegister} register The register to set.
 */
Device.prototype.setAsicRegister = function(register) {
    "use strict";
};
/**
 * This function sets the pen parameters in a device.
 * @param {PenParameters} penParams Pen parameters to set.
 */
Device.prototype.setPenParameters = function(penParams) {
    "use strict";
};
/**
 * This function sets the touch parameters in a device.
 * @param {TouchParameters} touchParams Touch parameters to set.
 */
Device.prototype.setTouchParameters = function(touchParams) {
    "use strict";
};
/**
 * This function sets a region as the current region in the device.
 * @param {number} regionNumber The region number to set as primary.
 */
Device.prototype.setPrimaryRegion = function(regionNumber) {
    "use strict";
};
/**
 * This method writes a display chunk tto the display of a device.
 * @param {DisplayChunk} chunk Chunk to write.
 */
Device.prototype.writeDisplayChunk = function(chunk) {
    "use strict";
};
/**
 * Writes a region to the device.
 * @param {LegacyRegion} region Region to write.
 */
Device.prototype.setRegion = function(region) {
    "use strict";
};
/**
 * This function updates a region.
 * @param {RegionUpdate} update The update to apply.
 */
Device.prototype.updateRegion = function(update) {
    "use strict";
};
/**
 * This function either saves or discards region changes.
 * @param {boolean} save If true saves region changes in flash, false discards.
 */
Device.prototype.saveRegions = function(save) {
    "use strict";
};
/**
 * This function forces a device into calibration mode.
 */
Device.prototype.calibrate = function() {
    "use strict";
};
/**
 * Draws a text string to the display of the device.
 * @param {string} text Text to draw.
 * @param {number} x The horizontal position to start at.
 * @param {number} y The vertical position to start at.
 * @param {string} alignment LEFT, RIGHT or CENTER
 * @param {boolean} invert Whether or not to invert the text.
 */
Device.prototype.drawText = function(text, x, y, alignment, invert) {
    "use strict";
};
/**
 * Measures the text width of a string to be drawn with drawText().
 * @param {string} text Text to measure the width of.
 * @returns {number} Width in pixels of the text.
 */
Device.prototype.getTextWidth = function() {
    "use strict";
};
/**
 * Measures the text height of a string to be drawn with drawText().
 * @param {string} text Text to measure the height of.
 * @returns {number} Height in pixels of the text.
 */
Device.prototype.getTextHeight = function(text) {
    "use strict";
};

/**
 * Clears the last error returned as part of getDeviceStatus().
 */
Device.prototype.clearLastError = function() {
    "use strict";
};

/**
 * Gets the current resource retrieval index.
 * @returns {number} The current resource retrieval index.
 */
Device.prototype.getResourceRetrievalIndex = function() {
    "use strict";
};

/**
 * Sets the current resource retrieval index.
 * @param {number} index The index to set.
 */
Device.prototype.setResourceRetrievalIndex = function(index) {
    "use strict";
};

/**
 * Gets information about the resource at the current resource index.
 * @return {ResourceInfo} Information about the resource at the current resource index.
 */
Device.prototype.getResourceInfo = function() {
    "use strict";
};

/**
 * Gets information about the available space left in the device for resources.
 * @return {ResourceAvailability} Information about the available space left in the device for resources.
 */
Device.prototype.getResourceAvailability = function() {
    "use strict";
};

/**
 * Gets the resource at the current resource index.
 * @return {Resource} Resource at the current resource index.
 */
Device.prototype.getResource = function() {
    "use strict";
};

/**
 * Stores a resource in the flash of the firmware.
 * @param {Resource} resource Resource to save.
 */
Device.prototype.setResource = function(resource) {
    "use strict";
};

/**
 * Deletes a resource at a particular index.
 * @param {number} index Index to delete.
 */
Device.prototype.deleteResource = function(index) {
    "use strict";
};

/**
 * Gets the current region retrieval index.
 * @returns {number} The current index.
 */
Device.prototype.getRegionRetrievalIndex = function() {
    "use strict";
};

/**
 * Sets the current region retrieval index.
 * @param {number} index The index to set.
 */
Device.prototype.setRegionRetrievalIndex = function(index) {
    "use strict";
};

/**
 * Gets the region at the current index.
 * @return {Region} Region at the current index.
 */
Device.prototype.getDeviceRegion = function() {
    "use strict";
};

/**
 * Gets a complete list of regions from the device.
 * @return {Array<Region>} List of regions from the device.
 */
Device.prototype.getDeviceRegions = function() {
    "use strict";
};

/**
 * Saves a region to the device.
 * @param {Region} Region to save.
 */
Device.prototype.setDeviceRegion = function(region) {
    "use strict";
};

/**
 * Deletes a region at a particular index.
 * @param {number} index Index to delete.
 */
Device.prototype.deleteDeviceRegion = function(index) {
    "use strict";
};

/**
 * Gets the current screen retrieval index.
 * @return {number} The current index.
 */
Device.prototype.getScreenRetrievalIndex = function() {
    "use strict";
};

/**
 * Sets the current screen retrieval index.
 * @param {number} index The index to set.
 */
Device.prototype.setScreenRetrievalIndex = function() {
    "use strict";
};

/**
 * Gets the screen at the current index.
 * @return {Screen} Screen at the current index.
 */
Device.prototype.getScreen = function() {
    "use strict";
};

/**
 * Sets a screen in the device.
 * @param {Screen} screen Screen to write.
 */
Device.prototype.setScreen = function(screen) {
    "use strict";
};

/**
 * Deletes a screen at a particular index.
 * @param {number} Index to delete.
 */
Device.prototype.deleteScreen = function() {
    "use strict";
};

/**
 * Modifies a screen.
 * @param {ScreenModify} modify The parameters for modifying a screen.
 */
Device.prototype.modifyScreen = function(modify) {
    "use strict";
};

/**
 * Gets the screen, region, resource and other associated
 * limits from the device.
 * @return {ScreenRegionInfo} Information about limitations around regions.
 */
Device.prototype.getScreenRegionInfo = function() {
    "use strict";
};

/**
 * Gets the current active screen.
 * @return {number} Currently active screen.
 */
Device.prototype.getActiveScreen = function() {
    "use strict";
};

/**
 * Sets the current active screen.
 * @param {number} index Screen index to set.
 */
Device.prototype.setActiveScreen = function(index) {
    "use strict";
};

/**
 * This method allows you to save, discard, or restore to factory
 * defaults the screen settings in the device.
 * @param {string} action Action to perform, either: DISCARD, SAVE, or FACTORY_DEFAULTS
 */
Device.prototype.saveScreensRegions = function(action) {
    "use strict";
};

/**
 * This class contains information about the amount of resources
 * available for screen editing.
 * @constructor
 * @class
 */
function ScreenRegionInfo() {
    "use strict";
    /**
     * The number of screens supported by this device.
     * @type {number}
     */
    this.screenCount = 0;
    /**
     * The number of regions supported by this device.
     * @type {number}
     */
    this.regionCount = 0;
    /**
     * The number of regions per screen supported by this device.
     * @type {number}
     */
    this.regionsPerScreen = 0;
    /**
     * The number of resources supported by this device.
     * @type {number}
     */
    this.resourceCount = 0;
    /**
     * The maximum caption length.
     * @type {number}
     */
    this.captionLength = 0;
    /**
     * The maximum text region length;
     * @type {number}
     */
    this.textRegionLength = 0;
    /**
     * The width of one character in the device.
     * @type {number}
     */
    this.textWidth = 0;
    /**
     * The height of one character in the device.
     * @type {number}
     */
    this.textHeight = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved1 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved2 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved3 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved4 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved5 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved6 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved7 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved8 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved9 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved10 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved11 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved12 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved13 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved14 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved15 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved16 = 0;
}

/**
 * This class contains information about a screen stored in the device.
 * @constructor
 * @class
 */
function ScreenModify() {
    "use strict";
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved1 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved2 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved3 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved4 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved5 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved6 = 0;
    /**
     * The screen number to modify.
     * @type {number}
     */
    this.screen = 0;
    /**
     * If this is an object, it adds a region to a screen.
     * @type {object}
     */
    this.add = {
        /**
         * Region to add to screen.
         * @type {number}
         */
        region: 0,
        /**
         * Next screen to go to when advancing from this region. 0xFF for no action.
         * @type {number}.
         */
        nextScreen: 0
    };
    /**
     * If this is an object, it deletes a region from a screen.
     * @type {object}
     */
    this.delete = {
        /**
         * Region to delete from screen.
         * @type {number}
         */
        region: 0
    };
    /**
     * If this is an object, sets timeout properties for a screen.
     * @type {object}
     */
    this.timeout = {
        /**
         * Timeout in seconds, 0 disables.
         * @type {number}
         */
        timeout: 0,
        /**
         * The screen to advance to when the timeout is done.
         * @type {number}
         */
        timeoutScreenNext: 0
    };
}

/**
 * This class contains information about a screen stored in the device.
 * @constructor
 * @class
 */
function Screen() {
    "use strict";
    /**
     * The screen number.
     * @type {number}
     */
    this.number = 0;
    /**
     * Whether or not this screen timeout is reset if the pen goes down.
     * @type {boolean}
     */
    this.resetTimeoutOnActivity = false;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved1 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved2 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved3 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved4 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved5 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved6 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved7 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved8 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved9 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved10 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved11 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved12 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved13 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved14 = 0;
    /**
     * The timeout of this screen in seconds.
     */
    this.timeout = 0;
    /**
     * The screen we should move on to if the timeout is hit. 0xFF for no change.
     */
    this.timeoutScreenNext = 0;
    /**
     * A list of regions to send to the device that are associated with this screen.
     */
    this.regions = [];
    /**
     * A list of screens we'll move on to if the corrisponding region index is activated.
     * This list must always match regions.length exactly. 0xFF for no change.
     */
    this.screensNext = [];
}

/**
 * This class contains information about a resource stored in the device.
 * @constructor
 * @class
 */
function Resource() {
    "use strict";
    /**
     * The resource number.
     * @type {number}
     */
    this.number = 0;
    /**
     * The resource type, either GRAPHIC, TEXT, or INVALID.
     * @type {string}
     */
    this.type = "INVALID";
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved1 = 0;
    /**
     * The size of this resource in bytes.
     * @type {number} 
     */
    this.size = 0;
    /**
     * The CRC32 of this resource.
     * @type {number}
     */
    this.crc = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved2 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved3 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved4 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved5 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved6 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved7 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved8 = 0;
    /**
     * The raw data stored in the resource.
     * @type {Array<number>}
     */
    this.data = [];
}

/**
 * This class contains information about the amount of storage left for resources.
 * @constructor
 * @class
 */
function ResourceAvailability() {
    "use strict";
    /**
     * The number of regions still available for use.
     * @type {number}
     */
    this.available = 0;
    /**
     * The number of bytes still available for use.
     * @type {number}
     */
    this.bytesAvailable = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved1 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved2 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved3 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved4 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved5 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved6 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved7 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved8 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved9 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved10 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved11 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved12 = 0;
}

/**
 * This class contains information about a resource stored in
 * flash.
 * @constructor
 * @class
 */
function ResourceInfo() {
    "use strict";
    /**
     * The resource number.
     * @type {number}
     */
    this.number = 0;
    /**
     * The resource type, either GRAPHIC, TEXT, or INVALID.
     * @type {string}
     */
    this.type = "INVALID";
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved1 = 0;
    /**
     * The size of this resource in bytes.
     * @type {number} 
     */
    this.size = 0;
    /**
     * The CRC32 of this resource.
     * @type {number}
     */
    this.crc = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved2 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved3 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved4 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved5 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved6 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved7 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved8 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.reserved9 = 0;
}

/**
 * This class contains information about the sample taken from a coordinate
 * event.
 * @constructor
 * @class
 */
function SampleInfo() {
    "use strict";
    /**
     * Whether or not the pen tip was pressed.
     * @type {boolean}
     */
    this.down = false;
    /**
     * The data source of the event.
     * @type {number}
     */
    this.dataSource = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved1 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved2 = 0;
}

/**
 * This class contains information about a coordinate event sent by a ScripTouch
 * device.
 * @constructor
 * @class
 */
function InputCoordinateEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "Coordinate";
    /**
     * The horizontal component of the sample.
     * @type {number}
     */
    this.x = 0;
    /**
     * The vertical component of the sample.
     * @type {number}
     */
    this.y = 0;
    /**
     * Information about the sample.
     * @type {SampleInfo}
     */
    this.sampleInfo = new SampleInfo();
}

/**
 * This class contains information about a coordinate event sent by a ScripTouch
 * device.
 * @constructor
 * @class
 */
function ExtendedInputCoordinateEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "ExtendedCoordinate";
    /**
     * The horizontal component of the sample.
     * @type {number}
     */
    this.x = 0;
    /**
     * The vertical component of the sample.
     * @type {number}
     */
    this.y = 0;
    /**
     * The region number the pen is currently on or over.
     * @type {number}
     */
    this.region = 0;
    /**
     * Information about the sample.
     * @type {SampleInfo}
     */
    this.sampleInfo = new SampleInfo();
}

/**
 * This class represents a magnetic card swipe event generated by a device.
 * @constructor
 * @class
 */
function MagCardDataEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "MagCardData";
    /**
     * The format of this card, either: 'Financial', 'Identification' or 'Unknown'.
     * @type {string}
     */
    this.format = "";
    /**
     * The status of the first track, either: 'Valid', 'ParityError', 'UnknownFormat', 'Corrupt', or "Unknown'.
     * @type {string}
     */
    this.track1Status = "";
    /**
     * The status of the second track, either: 'Valid', 'ParityError', 'UnknownFormat', 'Corrupt', or "Unknown'.
     * @type {string}
     */
    this.track2Status = "";
    /**
     * The status of the third track, either: 'Valid', 'ParityError', 'UnknownFormat', 'Corrupt', or "Unknown'.
     * @type {string}
     */
    this.track3Status = "";
    /**
     * The contents of the first track.
     * @type {string}
     */
    this.track1 = "";
    /**
     * The contents of the second track.
     * @type {string}
     */
    this.track2 = "";
    /**
     * The contents of the third track.
     * @type {string}
     */
    this.track3 = "";
}

/**
 * This class represents a button being depressed on the device.
 * @constructor
 * @class
 */
function ButtonDownEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "ButtonDown";
    /**
     * The horizontal component of the button press.
     * @type {number}
     */
    this.x = 0;
    /**
     * The vertical component of the button press.
     * @type {number}
     */
    this.y = 0;
    /**
     * The button region that was depressed.
     * @type {ButtonRegion}
     */
    this.region = {};
}

/**
 * This class represents a pen moving inside a button on the device.
 * @constructor
 * @class
 */
function ButtonMoveEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "ButtonMove";
    /**
     * The horizontal component of the button move.
     * @type {number}
     */
    this.x = 0;
    /**
     * The vertical component of the button move.
     * @type {number}
     */
    this.y = 0;
    /**
     * The button region that was moved inside of.
     * @type {ButtonRegion}
     */
    this.region = {};
}

/**
 * This class represents a pen that moved out of a button region and came up
 * thereby cancelling the button press.
 * @constructor
 * @class
 */
function ButtonUpEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "ButtonUp";
    /**
     * The horizontal component of the button up.
     * @type {number}
     */
    this.x = 0;
    /**
     * The vertical component of the button up.
     * @type {number}
     */
    this.y = 0;
    /**
     * The button region that came up.
     * @type {ButtonRegion}
     */
    this.region = {};
}

/**
 * This class represents an event that caused a button press on the device.
 * @constructor
 * @class
 */
function ButtonPressEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "ButtonPress";
    /**
     * The horizontal component of the button press.
     * @type {number}
     */
    this.x = 0;
    /**
     * The vertical component of the button press.
     * @type {number}
     */
    this.y = 0;
    /**
     * The button region that was pressed.
     * @type {ButtonRegion}
     */
    this.region = {};
}

/**
 * This class represents an ADC scan from the device. Not currently implemented.
 * @constructor
 * @class
 */
function AdcValuesEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "AdcValues";
}

/**
 * This class represents a mouse coordinate event from the device. Not currently implemented.
 * @constructor
 * @class
 */
function MouseCoordinateEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "MouseCoordinate";
}

/**
 * This class represents a digitizer pen event from the device. Not currently
 * implemented.
 * @constructor
 * @class
 */
function DigitizerPenCoordinateEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "DigitizerPenCoordinate";
}

/**
 * This class represents a touch event from the device. Not currently implemented.
 * @constructor
 * @class
 */
function DigitizerTouchCoordinateEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "DigitizerTouchCoordinate";
}

/**
 * This class represents a keyboard event from the device. Not currently implemented.
 * @constructor
 * @class
 */
function KeyboardEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "Keyboard";
}

/**
 * This class represents a debug coordinate event from the device. Not currently
 * implemented.
 * @constructor
 * @class
 */
function DebugCoordinateEvent() {
    "use strict";
    /**
     * The type of this event.
     * @type {string}
     */
    this.type = "DebugCoordinate";
}

/**
 * This class contains a set of device capabilities read from the device.
 * @constructor
 * @class
 */
function Capabilities() {
    /**
     * The type of display this device is equipped with, either:
     * 240X64_1X5_BLACKWHITE, 240X64_1X5_GRAYSCALE, NONE, or UNKNOWN.
     * @type {string}
     */
    this.displayId = "";
    /**
     * The type of card reader this device is equipped with, either:
     * HID, KEYBOARD_BASIC, KEYBOARD_PREAMBLE, NONE, or UNKNOWN.
     * @type {string}
     */
    this.magReaderId = "";
    /**
     * The type of reports this device supports, contains zero or more of these:
     * HID, MOUSE, PEN, TOUCH, KEYBOARD.
     * @type {Array<string>}
     */
    this.supportedReports = "";
    /**
     * The type of coordinate modes this device support, contains zero or more
     * of the following: STREAMING, BUFFERING, COMPRESSION, or THROTTLING.
     * @type {Array<string>}
     */
    this.coordinateModes = "";
    /**
     * The number of brightness levels this device supports.
     * @type {number}
     */
    this.brightnessLevels = 0;
    /**
     * The number of contrast levels this device supports.
     * @type {number}
     */
    this.contrastLevels = 0;
    /**
     * The color depth in bits of the display of this device.
     * @type {number}
     */
    this.colorDepth = 0;
    /**
     * The display width of this device in pixels.
     * @type {number}
     */
    this.displayWidth = 0;
    /**
     * The display height of this device in pixels.
     * @type {number}
     */
    this.displayHeight = 0;
    /**
     * The width of the digitizier in this device in millimeters.
     * @type {number}
     */
    this.digitizerWidth = 0;
    /**
     * The height of the digitizer in this device in millimeters.
     * @type {number}
     */
    this.digitizerHeight = 0;
    /**
     * The minimum horizontal component of any coordinate.
     * @type {number}
     */
    this.coordinateXMin = 0;
    /**
     * The maximum horizontal component of any coordinate.
     * @type {number}
     */
    this.coordinateXMax = 0;
    /**
     * The minimum vertical component of any coordinate.
     * @type {number}
     */
    this.coordinateYMin = 0;
    /**
     * The maximum vertical component of any coordinate.
     * @type {number}
     */
    this.coordinateYMax = 0;
	/**
	 * The minimum x value in the firmware's coordinate space
     * @type {number}
	 */
	this.calibrationXMin = 0;
	/**
	 * The maximum x value in the firmware's coordinate space
     * @type {number}
	 */
	this.calibrationXMax = 0;
	/**
	 * The minimum y value in the firmware's coordinate space
     * @type {number}
	 */
	this.calibrationYMin = 0;
	/**
	 * The maximum y value in the firmware's coordinate space
     * @type {number}
	 */
	this.calibrationYMax = 0;
 
 	/**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved4 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved5 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved6 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved7 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved8 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved9 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved10 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved11 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved12 = 0;
}

/**
 * This class contains information about the display of a device.
 * @constructor
 * @class
 */
function DisplayInfo() {
    "use strict";
    /**
     * The width of the display in pixels.
     * @type {number}
     */
    this.width = 0;
    /**
     * The height of the display in pixels.
     * @type {number}
     */
    this.height = 0;
    /**
     * The color depth of the display in bits.
     * @type {number}
     */
    this.colorDepth = 0;
    /**
     * The width of a single character in pixels.
     * @type {number}
     */
    this.textWidth = 0;
    /**
     * The height of a single character in pixels.
     * @type {number}
     */
    this.textHeight = 0;
    /**
     * The number of regions in this device.
     * @type {number}
     */
    this.regionCount = 0;
    /**
     * The maximum caption length of any button.
     * @type {number}
     */
    this.maxCaptionLength = 0;
}

/**
 * This class contains configuration settings about a device display.
 * @constructor
 * @class
 */
function DisplaySettings() {
    "use strict";
    /**
     * The current brightness setting.
     * @type {number}
     */
    this.brightness = 0;
    /**
     * The current contrast setting.
     * @type {number}
     */
    this.contrast = 0;
    /**
     * The red adjustment color.
     * @type {number}
     */
    this.red = 0;
    /**
     * The green adjustment color.
     * @type {number}
     */
    this.green = 0;
    /**
     * The blue adjustment color.
     * @type {number}
     */
    this.blue = 0;
    /**
     * If this is non-zero it will trigger the device to go into command mode.
     * @type {number}
     */
    this.setCommand = 0;
}

/**
 * This class represents the coordinate range the device is configured for.
 * @constructor
 * @class
 */
function CoordinateRange() {
    "use strict";
    /**
     * The minimum horizontal coordinate.
     * @type {number}
     */
    this.xMin = 0;
    /**
     * The maximum horizontal coordinate.
     * @type {number}
     */
    this.xMax = 0;
    /**
     * The minimum vertical coordinate.
     * @type {number}
     */
    this.yMin = 0;/**
     * The maximum vertical coordinate.
     * @type {number}
     */
    this.yMax = 0;
}

/**
 * This class contains the firmware version of the device.
 * @constructor
 * @class
 */
function VersionInfo() {
    "use strict";
    /**
     * The major version of the firmware.
     * @type {number}
     */
    this.firmwareMajor = 0;
    /**
     * The minor version of the firmware.
     * @type {number}
     */
    this.firmwareMinor = 0;
    /**
     * The release version of the firmware.
     * @type {number}
     */
    this.firmwareRelease = 0;
    /**
     * The major version of the kernel.
     * @type {number}
     */
    this.kernelMajor = 0;
    /**
     * The minor version of the kernel.
     * @type {number}
     */
    this.kernelMinor = 0;
    /**
     * The release version of the kernel.
     * @type {number}
     */
    this.kernelRelease = 0;
    /**
     * The asic signature of the embedded asic.
     * @type {number}
     */
    this.asicSignature = 0;
}

/**
 * This class represents a region in firmware.
 * @constructor
 * @class
 */
function Region() {
    "use strict";
    /**
     * The type of this region, either: CONTAINER, LINE, BUTTON, GRAPHIC, TEXT, NULL or UNKNOWN.
     * @type {string}
     */
    this.type = "";
    /**
     * The number of this region, starting with 1 and ending with region max from display info.
     * @type {number}
     */
    this.regionNumber = 0;
    /**
     * Whether or not this region should accept touches on capable devices.
     * @type {boolean}
     */
    this.acceptTouch = false;
    /**
     * Whether or not this region should be considered when figuring out which
     * region the pen is currently drawing in.
     * @type {boolean}
     */
    this.decode = false;
    /**
     * Whether or not a 1px border should be drawn around the region bondaries.
     * @type {boolean}
     */
    this.hasFrame = false;
    /**
     * Whether or not to output coordinates to the host for this region.
     * @type {boolean}
     */
    this.outputToHost = false;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved1 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved2 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved3 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved4 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved5 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved6 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved7 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved8 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved9 = 0;
    /**
     * The left-most component of the region boundary.
     */
    this.x1 = 0;
    /**
     * The right-most component of the region boundary.
     */
    this.x2 = 0;
    /**
     * The top-most component of the region boundary.
     */
    this.y1 = 0;
    /**
     * The bottom-most component of the region boundary.
     */
    this.y2 = 0;
    /**
     * The color of the foreground of this device.
     * @type {Color}
     */
    this.colorForeground = new Color();
    /**
     * The color of the background of this device.
     * @type {Color}
     */
    this.colorBackground = new Color();
    
    //Container
    /**
     * For container types this will enable "inking" in this region.
     * @type {boolean}
     */
    this.inkEnable = false;
    /**
     * The width of the line made by ink.
     * @type {number}
     */
    this.inkWidth = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.containerReserved1 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.containerReserved2 = 0;
    
    //Line
    /**
     * For line regions, the width of the line.
     * @param {number}
     */
    this.width = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.lineReserved = 0;
    
    //Button
    /**
     * For button regions should we enable the button?
     * @type {boolean}
     */
    this.enable = false;
    /**
     * For button regions should the button be highlighted on select?
     * @type {boolean}
     */
    this.highlightOnSelect = false;
    /**
     * For button regions should the screen be cleared when the button is pressed?
     * @type {boolean}
     */
    this.clearOnSelect = false;
    /**
     * For button regions should the signature be confirmed when the button is pressed?
     * @type {boolean}
     */
    this.confirmOnSelect = false;
    /**
     * For this button, should resources be used (true), or should a caption (false)?
     * @type {boolean}
     */
    this.useResource = false;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.buttonReserved1 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.buttonReserved2 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.buttonReserved3 = 0;
    /**
     * For button regions this should contain the caption for the button.
     * @type {string}
     */
    this.caption = "";
    /**
     * If useResources is true, this is the resource index of the front side of the button.
     * @type {number}
     */
    this.resourceNormal = 0;
    /**
     * If useResources is true, this is the resource index of the back side of the button, or 0 for no change.
     * @type {number}
     */
    this.resourceSelected = 0;

    //Graphic/Text
    /** 
     * For graphics and text this sets the horizontal justification, either: LEFT, CENTER, RIGHT.
     * @type {string}
     */
    this.horizontalJustification = "LEFT";
    /**
     * For graphics and text this sets the vertical justification, either: TOP, CENTER, BOTTOM.
     * @type {string}
     */
    this.verticalJustification = "TOP";

    //Graphic
    /**
     * The resource index of this graphic.
     * @type {number}
     */
    this.graphicResource = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.graphicReserved1 = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.graphicReserved2 = 0;

    //Text
    /**
     * The resource index of this text region.
     * @type {number}
     */
    this.textResource = 0;
    /**
     * Reserved for future use.
     * @type {number}
     */
    this.textReserved1 = 0;
    /**
     * The font number, currently unused.
     * @type {number}
     */
    this.fontNumber = 0;
}

/**
 * This class represents a legacy region in firmware.
 * @constructor
 * @class
 */
function LegacyRegion() {
    "use strict";
    /**
     * The type of this region, either: CONTAINER, LINE, BUTTON, NULL or UNKNOWN.
     * @type {string}
     */
    this.type = "";
    /**
     * The number of this region, starting with 1 and ending with region max from display info.
     * @type {number}
     */
    this.regionNumber = 0;
    /**
     * Whether or not this region should accept touches on capable devices.
     * @type {boolean}
     */
    this.acceptTouch = false;
    /**
     * Whether or not this region should be considered when figuring out which
     * region the pen is currently drawing in.
     * @type {boolean}
     */
    this.decode = false;
    /**
     * Whether or not this region is visible.
     * @type {boolean}
     */
    this.visible = false;
    /**
     * Whether or not this region is currently being used.
     * @type {boolean}
     */
    this.inUse = false;
    /**
     * Whether or not a 1px border should be drawn around the region bondaries.
     * @type {boolean}
     */
    this.hasFrame = false;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved1 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved2 = 0;
    /**
     * The left-most component of the region boundary.
     */
    this.x1 = 0;
    /**
     * The right-most component of the region boundary.
     */
    this.x2 = 0;
    /**
     * The top-most component of the region boundary.
     */
    this.y1 = 0;
    /**
     * The bottom-most component of the region boundary.
     */
    this.y2 = 0;
    /**
     * The color of the foreground of this device.
     * @type {Color}
     */
    this.colorForeground = new Color();
    /**
     * The color of the background of this device.
     * @type {Color}
     */
    this.colorBackground = new Color();
    
    //Container
    /**
     * For container types this will enable "inking" in this region.
     * @type {boolean}
     */
    this.inkEnable = false;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.containerReserved1 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.containerReserved2 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.containerReserved3 = 0;
    
    //Line / Button
    /**
     * For lines and buttons this is the parent container that these elements
     * should be associated with.
     * @type {number}
     */
    this.parent = 0;
    
    //Line
    /**
     * For line regions, the width of the line.
     * @param {number}
     */
    this.width = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.lineReserved = 0;
    
    //Button
    /**
     * For button regions should we enable the button?
     * @type {boolean}
     */
    this.enable = false;
    /**
     * For button regions should the button be highlighted on select?
     * @type {boolean}
     */
    this.highlightOnSelect = false;
    /**
     * For button regions should the screen be cleared when the button is pressed?
     * @type {boolean}
     */
    this.clearOnSelect = false;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.buttonReserved = 0;
    /**
     * For button regions this should contain the caption for the button.
     * @type {string}
     */
    this.caption = "";
}

/**
 * This class represents a color.
 * @constructor
 * @class
 */
function Color() {
    "use strict";
    /**
     * Red component.
     * @type {number}
     */
    this.red = 0;
    /**
     * Green component.
     * @type {number}
     */
    this.green = 0;
    /**
     * Blue component.
     * @type {number}
     */
    this.blue = 0;
    /**
     * Alpha component.
     * @type {number}
     */
    this.alpha = 0;
}

/**
 * This class represents the operating modes of a device.
 * @constructor
 * @class
 */
function OperatingModes() {
    "use strict";
    /**
     * This is a collection of supported operating modes by the device, can include:
     * TOUCH, PEN, AUTOTOUCH, COMMAND, and UNKNOWN.
     * @type {Array<string>}
     */
    this.supportedModes = [];
    /**
     * The current acquisition mode of the device. Can be: TOUCH, PEN, AUTOTOUCH, COMMAND, UNKNOWN.
     * @type {string}
     */
    this.acquisitionMode = "";
    /**
     * The current pen mode of the device, can be: SINGLE, SINGLELIFTOFF, STREAM,
     * STREAMLIFTOFF, CONTINUOUS, or UNKNOWN.
     * @type {string}
     */
    this.penMode = "";
    /**
     * The current touch mode of this device, can be: SINGLE, SINGLELIFTOFF, STREAM,
     * STREAMLIFTOFF, CONTINOUS, UNKNOWN
     */
    this.touchMode = "";
}

/**
 * This class represents general input configuration for a device.
 * @constructor
 * @class
 */
function GeneralParameters() {
    "use strict";
    /**
     * This sets whether or not normal HID coordinate output is enabled or not.
     * @type {boolean}
     */
    this.xyOutputEnable = false;
    /**
     * This sets whether or not raw ADC output is enabled.
     * @type {boolean}
     */
    this.adcOutputEnable = false;
    /**
     * This sets whether or not coordinates should be mapped prior to output.
     * @type {boolean}
     */
    this.mapCoordinates = false;
    /**
     * This sets whether or not channel lock is enabled.
     * @type {boolean}
     */
    this.channelLockEnable = false;
    /**
     * This sets the channel lock number if channelLockEnable is true.
     * @type {number}
     */
    this.channelLockNumber = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved = 0;
    /**
     * The corner assigned to the lower left.
     * @type {number}
     */
    this.cornerLowerLeft = 0;
    /**
     * The corner assigned to the upper left.
     * @type {number}
     */
    this.cornerUpperLeft = 0;
    /**
     * The corner assigned to the lower right.
     * @type {number}
     */
    this.cornerLowerRight = 0;
    /**
     * The corner assigned to the upper right.
     * @type {number}
     */
    this.cornerUpperRight = 0;
    /**
     * The number of samples.
     * @type {number}
     */
    this.samples = 0;
    /**
     * The filter frequency.
     * @type {number}
     */
    this.filterFrequency = 0;
}

/**
 * This class represents pen input configuration for a device.
 * @constructor
 * @class
 */
function PenParameters() {
    "use strict";
    /**
     * Sets the required proximity.
     * @type {number}
     */
    this.proximity = 0;
    /**
     * Sets the auto pen threshold.
     * @type {number}
     */
    this.autoPenThreshold = 0;
    /**
     * Sets the auto touch threshold.
     * @type {number}
     */
    this.autoTouchThreshold = 0;
    /**
     * Sets the number of adc averages to take.
     * @type {number}
     */
    this.adcAverage = 0;
    /**
     * Sets the number of coordinate averages to take.
     * @type {number}
     */
    this.coordinateAverage = 0;
    /**
     * Sets the offet interval.
     * @type {number}
     */
    this.offsetInterval = 0;
    /**
     * Sets the debounce pre down.
     * @type {number}
     */
    this.debouncePreDown = 0;
    /**
     * Sets the debounce post down.
     * @type {number}
     */
    this.debouncePostDown = 0;
    /**
     * Sets the debounce pre up.
     * @type {number}
     */
    this.debouncePreUp = 0;
    /**
     * Sets the debounce post up.
     * @type {number}
     */
    this.debouncePostUp = 0;
    /**
     * Sets whether or not error correction should be applied to input samples.
     * @type {boolean}
     */
    this.errorCorrectionEnable = false;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved1 = 0;
    /**
     * Sets whether or not offset correction should be applied to input samples.
     * @type {boolean}
     */
    this.offsetCorrectionEnable = false;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved2 = 0;
    /**
     * Sets whether or not the tip switch should be inverted.
     * @type {boolean}
     */
    this.tipInverted = false;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved3 = 0;
    /**
     * Sets the interrupt rate on the device.
     * @type {number}
     */
    this.interruptRate = 0;
}

/**
 * This class represents touch input configuration for a device.
 * @constructor
 * @class
 */
function TouchParameters() {
    "use strict";
    /**
     * The number of adc averages to take for input samples.
     * @type {number}
     */
    this.adcAverage = 0;
    /**
     * The number of coordinate averages to take for input samples.
     * @type {number}
     */
    this.coordinateAverage = 0;
    /**
     * The slope down.
     * @type {number}
     */
    this.slopeDown = 0;
    /**
     * The slope liftoff.
     * @type {number}
     */
    this.slopeLiftoff = 0;
    /**
     * The slope minimum.
     * @type {number}
     */
    this.slopeMinimum = 0;
    /**
     * The pressure single.
     * @type {number}
     */
    this.pressureSingle = 0;
    /**
     * The pressure multiple.
     * @type {number}
     */
    this.pressureMultiple = 0;
    /**
     * The gain.
     * @type {number}
     */
    this.gain = 0;
    /**
     * Whether or not error correction should be applied.
     * @type {boolean}
     */
    this.errorCorrectionEnable = false;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved = 0;
    /**
     * The X Scale.
     * @type {number}
     */
    this.scaleX = 0;
    /**
     * The X offset.
     * @type {number}
     */
    this.offsetX = 0;
    /**
     * The Y scale.
     * @type {number}
     */
    this.scaleY = 0;
    /**
     * The Y offset.
     * @type {number}
     */
    this.offsetY = 0;
}

/**
 * This class represents embedded data captured during calibration to do
 * coordinate correction.
 * @constructor
 * @class
 */
function CalibrationData() {
    "use strict";
    /**
     * The calibration type.
     * @type {number}
     */
    this.calibrationType = 0;
    /**
     * Whether or not we're currently calibrated.
     * @type {boolean}
     */
    this.isCalibrated = false;
    /**
     * The rotation scale.
     * @type {number}
     */
    this.rotationScale = 0;
    /**
     * The local multiplier scale.
     * @type {number}
     */
    this.localMultiplierScale = 0;
    /**
     * The host multiplier scale.
     * @type {number}
     */
    this.hostMultiplierScale = 0;
    /**
     * The rotation sin scaled.
     * @type {number}
     */
    this.rotationSinScaled = 0;
    /**
     * The rotation cos scaled.
     * @type {number}
     */
    this.rotationCosScaled = 0;
    /**
     * The local width calibrated.
     * @type {number}
     */
    this.localWidthCalibrated = 0;
    /**
     * The local height calibrated.
     * @type {number}
     */
    this.localHeightCalibrated = 0;
    /**
     * The local x multiplier scaled.
     * @type {number}
     */
    this.localXMultiplierScaled = 0;
    /**
     * The local X offset.
     * @type {number}
     */
    this.localXOffset = 0;
    /**
     * The local Y multiplier scaled.
     * @type {number}
     */
    this.localYMultiplierScaled = 0;
    /**
     * The local y offset.
     * @type {number}
     */
    this.localYOffset;
    /**
     * The host x multiplier scaled.
     * @type {number}
     */
    this.hostXMultiplierScaled = 0;
    /**
     * The host x offset.
     * @type {number}
     */
    this.hostXOffset = 0;
    /**
     * The host y multiplier scaled.
     * @type {number}
     */
    this.hostYMultiplierScaled = 0;
    /**
     * The host y offset.
     * @type {number}
     */
    this.hostYOffset = 0;
}

/**
 * This class represents the current device operating status.
 * @constructor
 * @class
 */
function DeviceStatus() {
    "use strict";
    /**
     * Whether or not the device is currently starting up.
     * @type {boolean}
     */
    this.isInitializing = false;
    /**
     * Whether or not the device is currently in command mode.
     * @type {boolean}
     */
    this.isIdle = false;
    /**
     * Whether or not calibration is required.
     * @type {boolean}
     */
    this.calibrationRequired = false;
    /**
     * Whether or not we're currently attempting to calibrate.
     * @type {boolean}
     */
    this.isCalibrating = false;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved1 = 0;
    /**
     * Reserved for later use.
     * @type {number}
     */
    this.reserved2 = 0;
    /**
     * An error code for the last error.
     * @type {number}
     */
    this.lastError = 0;
    /**
     * A string containing a textual representation of the error.
     * @type {string}
     */
    this.errorInfo = "";
}

/**
 * This class represents output configuration for a device.
 * @constructor
 * @class
 */
function OutputConfiguration() {
    "use strict";
    /**
     * Output mode, can be one of the following: REAL_TIME, TRANSMIT_ON_CONFIRM,
     * TRANSMIT_ON_HOST_COMMAND, UNKNOWN
     * @type {string}
     */
    this.outputMode = "";
    /**
     * Compression mode, can be one of the following: UNCOMPRESSED_LORES,
     * COMPRESSED_LORES, UNCOMPRESSED_HIRES, COMPRESSED_HIRES
     * @type {string}
     */
    this.compression = "";
    /**
     * The number of packets per second coordinates should be sent at.
     * @type {number}
     */
    this.packetsPerSecond = 0;
    /**
     * Whether or not to save this as the default.
     * @type {boolean}
     */
    this.saveAsDefault = false;
    /**
     * Whether or not to load factory defaults.
     * @type {boolean}
     */
    this.loadFactorySettings = false;
}

/**
 * This class represents the frequency amplitude settings for the device.
 * @constructor
 * @class
 */
function FrequencyAmplitude() {
    "use strict";
    /**
     * The operating freqency.
     * @type {number}
     */
    this.frequency = 0;
    /**
     * The pen amplitude setting.
     * @type {number}
     */
    this.penAmplitude = 0;
    /**
     * The touch amplitude setting.
     * @type {number}
     */
    this.touchAmplitude = 0;
}

/**
 * This class represents the current calibration state.
 * @constructor
 * @class
 */
function CalibrationState() {
    "use strict";
    /**
     * Whether or not we're currently calibrating.
     * @type {boolean}
     */
    this.calibrationMode = false;
    /**
     * The point we're currently calibrating (if calibrationMode is true), can
     * be: CENTER, LOWERLEFT, LOWERRIGHT, UPPERLEFT, UPPERRIGHT, UNKNOWN.
     */
    this.point = "";
}

/**
 * This class represents the current configuration.
 * @constructor
 * @class
 */
function Configuration() {
    "use strict";
    /**
     * Whether or not the device is in a mode.
     * @type {boolean}
     */
    this.deviceMode = false;
    /**
     * The device id.
     * @type {number}
     */
    this.deviceId = 0;
}

/**
 * This class represents a firmware page.
 * @constructor
 * @class
 */
function FirmwarePage() {
    "use strict";
    /**
     * The address of the firmware page.
     * @type {number}
     */
    this.address = 0;
    /**
     * The checksum of the firmware page.
     * @type {number}
     */
    this.checksum = 0;
    /**
     * The data contained in the page, should be 128 elements long exactly.
     * @type {Array<number>}
     */
    this.data = [];
}

/**
 * This class represents a single ASIC register.
 * @constructor
 * @class
 */
function AsicRegister() {
    "use strict";
    /**
     * The register id to set.
     * @type {number}
     */
    this.registerId = 0;
    /**
     * The value to write.
     * @type {number}
     */
    this.registerValue = 0;
}

/**
 * This class represents a display chunk that can be written to the display of
 * a device.
 * @constructor
 * @class
 */
function DisplayChunk() {
    "use strict";
    /**
     * The horizontal location of the image to display.
     * @type {number}
     */
    this.x = 0;
    /**
     * The vertical location of the image to display.
     * @type {number}
     */
    this.y = 0;
    /**
     * The width of the chunk.
     * @type {number}
     */
    this.width = 0;
    /**
     * The height of the chunk.
     * @type {number}
     */
    this.height = 0;
    /**
     * The number of bits per pixel, right now only 1 and 4 are supported.
     * @type {number}
     */
    this.bitsPerPixel = 0;
    /**
     * The raw graphic raster, should be 2039 elements long.
     * @type {Array<number>}
     */
    this.graphic = [];
}

/**
 * This class represents a region update.
 * @constructor
 * @class
 */
function RegionUpdate() {
    "use strict";
    /**
     * The region number to update.
     * @type {number}
     */
    this.regionNumber = 0;
    /**
     * For buttons this enables/disables the button, for containers it turns inking on and off.
     * @type {boolean}
     */
    this.functionEnable = false;
    /**
     * Whether or not the region should be visible.
     * @type {boolean}
     */
    this.visible = false;
    /**
     * Reserved for future use.
     */
    this.reserved1 = 0;
    /**
     * Reserved for future use.
     */
    this.reserved2 = 0;
    /**
     * The foreground color the region should be updated with.
     * @type {Color}
     */
    this.colorForeground = new Color();
}